##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""
    Macro library containning scan macros for the macros server Tango device 
    server as part of the Sardana project.
   
   Available Macros are:
     ascan family: ascan, a2scan, a3scan, a4scan and amultiscan
     dscan family: dscan, d2scan, d3scan, d4scan and dmultiscan
     mesh
     fscan
     scanhist
"""

__all__ = ["a2scan", "a3scan", "a4scan", "amultiscan", "aNscan", "ascan",
           "d2scan", "d3scan", "d4scan", "dmultiscan", "dNscan", "dscan",
           "fscan", "mesh", 
           "a2scanc", "a3scanc", "a4scanc", "ascanc",
           "d2scanc", "d3scanc", "d4scanc", "dNScanc", "dscanc",
           "meshc", 
           "a2scanct", "a3scanct", "a4scanct", "ascanct",  
           "scanhist", "getCallable", "UNCONSTRAINED"]

__docformat__ = 'restructuredtext'

import os
import copy
import datetime

import numpy

from taurus.console import Alignment
from taurus.console.list import List
from taurus.console.table import Table

from sardana.macroserver.msexception import UnknownEnv
from sardana.macroserver.macro import *
from sardana.macroserver.scan import *
from sardana.util.motion import Motor, MotionPath
from sardana.util.tree import BranchNode, LeafNode, Tree

UNCONSTRAINED="unconstrained"

StepMode = 's'
ContinuousMode = 'c' #TODO: change it to be more verbose e.g. ContinuousSwMode
ContinuousHwTimeMode = 'ct'
HybridMode = 'h'

def getCallable(repr):
    '''returns a function .
    Ideas: repr could be an URL for a file where the function is contained,
    or be evaluable code, or a pickled function object,...
    
    In any case, the return from it should be a callable of the form:
    f(x1,x2) where x1, x2 are points in the moveable space and the return value
    of f is True if the movement from x1 to x2 is allowed. False otherwise'''
    if repr==UNCONSTRAINED:
        return lambda x1,x2:True
    else:
        return lambda: None

class aNscan(Hookable):
    
    hints = { 'scan' : 'aNscan', 'allowsHooks': ('pre-scan', 'pre-move', 'post-move', 'pre-acq', 'post-acq', 'post-step', 'post-scan') }
    #env = ('ActiveMntGrp',)
    
    """N-dimensional scan. This is **not** meant to be called by the user,
    but as a generic base to construct ascan, a2scan, a3scan,..."""
    def _prepare(self, motorlist, startlist, endlist, scan_length, integ_time, mode=StepMode, **opts):

        self.motors = motorlist
        self.starts = numpy.array(startlist,dtype='d')
        self.finals = numpy.array(endlist,dtype='d')
        self.mode = mode
        self.integ_time = integ_time
        self.opts = opts
        if len(self.motors) == self.starts.size == self.finals.size:
            self.N = self.finals.size
        else:
            raise ValueError('Moveablelist, startlist and endlist must all be same length')

        moveables = []
        for m, start, final in zip(self.motors, self.starts, self.finals):
            moveables.append(MoveableDesc(moveable=m, min_value=min(start,final), max_value=max(start,final)))
        moveables[0].is_reference = True

        env = opts.get('env',{})
        constrains = [getCallable(cns) for cns in opts.get('constrains',[UNCONSTRAINED])]
        extrainfodesc = opts.get('extrainfodesc',[])

        #Hooks are not always set at this point. We will call getHooks later on in the scan_loop
        #self.pre_scan_hooks = self.getHooks('pre-scan')
        #self.post_scan_hooks = self.getHooks('post-scan'
          
        if mode == StepMode:
            self.nr_interv = scan_length
            self.nr_points = self.nr_interv+1
            self.interv_sizes = ( self.finals - self.starts) / self.nr_interv
            self.name = opts.get('name','a%iscan'%self.N)
            self._gScan = SScan(self, self._stepGenerator, moveables, env, constrains, extrainfodesc)
        elif mode in [ContinuousMode, ContinuousHwTimeMode]:
            #TODO: probably not 100% correct, 
            #      the idea is to allow passing a list of waypoints
            if isinstance(endlist[0],list):
                self.waypoints = self.finals
            else:
                self.waypoints = [self.finals]
            self.nr_waypoints = len(self.waypoints)
            if mode == ContinuousMode: 
                self.slow_down = scan_length
                self.nr_waypoints = 2 #aNscans will only have two waypoints (the start and the final positions)
                self.way_lengths = ( self.finals - self.starts) / (self.nr_waypoints -1)
                self.name = opts.get('name','a%iscanc'%self.N)
                self._gScan = CSScan(self, self._waypoint_generator, self._period_generator, moveables, env, constrains, extrainfodesc)
            elif mode == ContinuousHwTimeMode:
                self.nr_of_points = scan_length
                self.name = opts.get('name','a%iscanct'%self.N)
                self._gScan = CTScan(self, self._waypoint_generator_hwtime, 
                                           moveables, 
                                           env, 
                                           constrains, 
                                           extrainfodesc)
        elif mode == HybridMode:
            self.nr_interv = scan_length
            self.nr_points = self.nr_interv+1
            self.interv_sizes = ( self.finals - self.starts) / self.nr_interv
            self.name = opts.get('name','a%iscanh'%self.N)
            self._gScan = HScan(self, self._stepGenerator, moveables, env, constrains, extrainfodesc)
        else:
            raise ValueError('invalid value for mode %s' % mode)
        
    def _stepGenerator(self):
        step = {}
        step["integ_time"] =  self.integ_time
        step["pre-move-hooks"] = self.getHooks('pre-move')
        step["post-move-hooks"] = self.getHooks('post-move')
        step["pre-acq-hooks"] = self.getHooks('pre-acq')
        step["post-acq-hooks"] = self.getHooks('post-acq') + self.getHooks('_NOHINTS_')
        step["post-step-hooks"] = self.getHooks('post-step')
        
        step["check_func"] = []
        for point_no in xrange(self.nr_points):
            step["positions"] = self.starts + point_no * self.interv_sizes
            step["point_id"] = point_no
            yield step
    
    def _waypoint_generator(self):
        step = {}
        step["pre-move-hooks"] = self.getHooks('pre-move')
        step["post-move-hooks"] = self.getHooks('post-move')
        step["check_func"] = []
        step["slow_down"] = self.slow_down
        for point_no in xrange(self.nr_waypoints):
            step["positions"] = self.starts + point_no * self.way_lengths
            step["waypoint_id"] = point_no
            yield step
            
    def _waypoint_generator_hwtime(self):
        #TODO: remove starts
        def calculate_positions(moveable_node, start, end):
            '''Function to calculate starting and ending positions on the physical 
            motors level. 
            :param moveable_node: (BaseNode) node representing a moveable. 
                                  Can be a BranchNode representing a PseudoMotor,
                                  or a LeafNode representing a PhysicalMotor).
            :param start: (float) starting position of the moveable
            :param end: (float) ending position of the moveable
        
            :return: (list<(float,float)>) a list of tuples comprising starting
                     and ending positions. List order is important and preserved.'''
            start_positions = []
            end_positions = []
            if isinstance(moveable_node, BranchNode):
                pseudo_node = moveable_node
                moveable = pseudo_node.data
                moveable_nodes = moveable_node.children
                starts = moveable.calcPhysical(start)
                ends = moveable.calcPhysical(end)
                for moveable_node, start, end in zip(moveable_nodes, starts,
                                                        ends):
                    _start_positions, _end_positions = calculate_positions(
                                                                moveable_node, 
                                                                start, end)
                    start_positions += _start_positions
                    end_positions += _end_positions
            else:
                start_positions = [start]
                end_positions = [end]
                
            return start_positions, end_positions
        
        #CScan in its constructor populates a list of data structures - trees. 
        #Each tree represent one Moveables with its hierarchy of inferior moveables.                                                
        moveables_trees = self._gScan.get_moveables_trees()  
        step = {}
        step["pre-move-hooks"] = self.getHooks('pre-move')
        step["post-move-hooks"] = self.getHooks('post-move')
        step["check_func"] = []
        step["active_time"] = self.nr_of_points * self.integ_time
        step["positions"] = []
        step["start_positions"] = []
        starts = self.starts
        for point_no, waypoint in enumerate(self.waypoints):
            for start, end, moveable_tree in zip(starts, waypoint, 
                                                 moveables_trees):
                moveable_root = moveable_tree.root()
                start_positions, end_positions = calculate_positions(moveable_root, start, end)
                step["start_positions"] += start_positions
                step["positions"] += end_positions
                step["waypoint_id"] = point_no
                starts = waypoint
            yield step
        
    def _period_generator(self):
        step = {}
        step["integ_time"] =  self.integ_time
        step["pre-acq-hooks"] = self.getHooks('pre-acq')
        step["post-acq-hooks"] = self.getHooks('post-acq')+self.getHooks('_NOHINTS_')
        step["post-step-hooks"] = self.getHooks('post-step')
        step["check_func"] = []
        step['extrainfo'] = {}
        point_no = 0
        while(True):
            point_no += 1
            step["point_id"] = point_no
            yield step
    
    def run(self,*args):
        for step in self._gScan.step_scan():
            yield step
    
    @property
    def data(self):
        return self._gScan.data
    
    def getTimeEstimation(self):
        gScan = self._gScan
        mode = self.mode
        it = gScan.generator()
        v_motors = gScan.get_virtual_motors()
        curr_pos = gScan.motion.readPosition()
        total_time = 0.0
        if mode == StepMode:
            # calculate motion time
            max_step0_time, max_step_time = 0.0, 0.0
            # first motion takes longer, all others should be "equal"
            step0 = it.next()
            for v_motor, start, stop, length in zip(v_motors, curr_pos, step0['positions'], self.interv_sizes):
                path0 = MotionPath(v_motor, start, stop)
                path = MotionPath(v_motor, 0, length)
                max_step0_time = max(max_step0_time, path0.duration) 
                max_step_time = max(max_step_time, path.duration)
            motion_time = max_step0_time + self.nr_interv * max_step_time
            # calculate acquisition time
            acq_time = self.nr_points * self.integ_time
            total_time = motion_time + acq_time

        elif mode == ContinuousMode:
            total_time = gScan.waypoint_estimation()
        #TODO: add time estimation for ContinuousHwTimeMode
        return total_time
                        
    def getIntervalEstimation(self):
        mode = self.mode
        if mode == StepMode:
            return self.nr_interv
        elif mode == ContinuousMode:
            return self.nr_waypoints

class dNscan(aNscan):
    '''same as aNscan but it interprets the positions as being relative to the
    current positions and upon completion, it returns the motors to their
    original positions'''

    hints = copy.deepcopy(aNscan.hints)
    hints['scan'] = 'dNscan'

    def _prepare(self, motorlist, startlist, endlist, scan_length, integ_time, mode=StepMode, **opts):
        self._motion=self.getMotion( [ m.getName() for m in motorlist] )
        self.originalPositions = numpy.array(self._motion.readPosition())
        starts = numpy.array(startlist, dtype='d') + self.originalPositions
        finals = numpy.array(endlist, dtype='d') + self.originalPositions
        aNscan._prepare(self, motorlist, starts, finals, scan_length, integ_time, mode=mode, **opts)
        
    def do_restore(self):
        self.info("Returning to start positions...")
        self._motion.move(self.originalPositions)


class ascan(aNscan, Macro): 
    """Do an absolute scan of the specified motor.
    ascan scans one motor, as specified by motor. The motor starts at the
    position given by start_pos and ends at the position given by final_pos.
    The step size is (start_pos-final_pos)/nr_interv. The number of data points collected
    will be nr_interv+1. Count time is given by time which if positive,
    specifies seconds and if negative, specifies monitor counts. """

    param_def = [
       ['motor',      Type.Moveable,   None, 'Moveable to move'],
       ['start_pos',  Type.Float,   None, 'Scan start position'],
       ['final_pos',  Type.Float,   None, 'Scan final position'],
       ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
       ['integ_time', Type.Float,   None, 'Integration time']
    ]

    def prepare(self, motor, start_pos, final_pos, nr_interv, integ_time,
                **opts):
        self._prepare([motor], [start_pos], [final_pos], nr_interv, integ_time, **opts)
       

class a2scan(aNscan, Macro): 
    """two-motor scan.
    a2scan scans two motors, as specified by motor1 and motor2.
    Each motor moves the same number of intervals with starting and ending
    positions given by start_pos1 and final_pos1, start_pos2 and final_pos2,
    respectively. The step size for each motor is (start_pos-final_pos)/nr_interv.
    The number of data points collected will be nr_interv+1.
    Count time is given by time which if positive, specifies seconds and
    if negative, specifies monitor counts."""
    param_def = [
       ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
       ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
       ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
       ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
       ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
       ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
       ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
       ['integ_time', Type.Float,   None, 'Integration time']
    ]

    def prepare(self, motor1, start_pos1, final_pos1, motor2, start_pos2, final_pos2, nr_interv, integ_time,
                **opts):
        self._prepare([motor1,motor2], [start_pos1,start_pos2], [final_pos1,final_pos2], nr_interv, integ_time, **opts)


class a3scan(aNscan, Macro): 
    """three-motor scan .
    a3scan scans three motors, as specified by motor1, motor2 and motor3.
    Each motor moves the same number of intervals with starting and ending
    positions given by start_pos1 and final_pos1, start_pos2 and final_pos2,
    start_pos3 and final_pos3, respectively.
    The step size for each motor is (start_pos-final_pos)/nr_interv.
    The number of data points collected will be nr_interv+1.
    Count time is given by time which if positive, specifies seconds and
    if negative, specifies monitor counts."""
    param_def = [
       ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
       ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
       ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
       ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
       ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
       ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
       ['motor3',      Type.Moveable,   None, 'Moveable 3 to move'],
       ['start_pos3',  Type.Float,   None, 'Scan start position 3'],
       ['final_pos3',  Type.Float,   None, 'Scan final position 3'],
       ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
       ['integ_time', Type.Float,   None, 'Integration time']
    ]

    def prepare(self, m1, s1, f1,  m2, s2, f2, m3, s3, f3, nr_interv, integ_time,
                **opts):
        self._prepare([m1,m2,m3], [s1,s2,s3], [f1,f2,f3], nr_interv, integ_time, **opts)


class a4scan(aNscan, Macro): 
    """four-motor scan .
    a4scan scans four motors, as specified by motor1, motor2, motor3 and motor4.
    Each motor moves the same number of intervals with starting and ending
    positions given by start_posN and final_posN (for N=1,2,3,4).
    The step size for each motor is (start_pos-final_pos)/nr_interv.
    The number of data points collected will be nr_interv+1.
    Count time is given by time which if positive, specifies seconds and
    if negative, specifies monitor counts."""
    param_def = [
       ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
       ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
       ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
       ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
       ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
       ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
       ['motor3',      Type.Moveable,   None, 'Moveable 3 to move'],
       ['start_pos3',  Type.Float,   None, 'Scan start position 3'],
       ['final_pos3',  Type.Float,   None, 'Scan final position 3'],
       ['motor4',      Type.Moveable,   None, 'Moveable 3 to move'],
       ['start_pos4',  Type.Float,   None, 'Scan start position 3'],
       ['final_pos4',  Type.Float,   None, 'Scan final position 3'],
       ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
       ['integ_time', Type.Float,   None, 'Integration time']
    ]

    def prepare(self, m1, s1, f1, m2, s2, f2, m3, s3, f3, m4, s4, f4, nr_interv,
                integ_time, **opts):
        self._prepare([m1,m2,m3,m4], [s1,s2,s3,s4], [f1,f2,f3,f4], nr_interv, integ_time, **opts)


class amultiscan(aNscan, Macro): 
    '''Multiple motor scan.
    amultiscan scans N motors, as specified by motor1, motor2,...,motorN.
    Each motor moves the same number of intervals with starting and ending
    positions given by start_posN and final_posN (for N=1,2,...).
    The step size for each motor is (start_pos-final_pos)/nr_interv.
    The number of data points collected will be nr_interv+1.
    Count time is given by time which if positive, specifies seconds and
    if negative, specifies monitor counts."""
    '''

    param_def = [
       ['motor_start_end_list',
        ParamRepeat(['motor', Type.Moveable, None, 'Moveable to move'],
                    ['start',   Type.Float, None, 'Starting position'],
                    ['end',   Type.Float, None, 'Final position']),
        None, 'List of motor, start and end positions'],
       ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
       ['integ_time', Type.Float,   None, 'Integration time']
    ]
    
    def prepare(self, *args, **opts):
        motors = args[0:-2:3]
        starts = args[1:-2:3]
        ends   = args[2:-2:3]
        nr_interv = args[-2]
        integ_time = args[-1]
        
        self._prepare(motors, starts, ends, nr_interv, integ_time, **opts)

class dmultiscan(dNscan, Macro): 
    '''Multiple motor scan relative to the starting positions.
    dmultiscan scans N motors, as specified by motor1, motor2,...,motorN.
    Each motor moves the same number of intervals If each motor is at a
    position X before the scan begins, it will be scanned from X+start_posN
    to X+final_posN (where N is one of 1,2,...)
    The step size for each motor is (start_pos-final_pos)/nr_interv.
    The number of data points collected will be nr_interv+1.
    Count time is given by time which if positive, specifies seconds and
    if negative, specifies monitor counts."""
    '''
    param_def = [
       ['motor_start_end_list',
        ParamRepeat(['motor', Type.Moveable, None, 'Moveable to move'],
                    ['start',   Type.Float, None, 'Starting position'],
                    ['end',   Type.Float, None, 'Final position']),
        None, 'List of motor, start and end positions'],
       ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
       ['integ_time', Type.Float,   None, 'Integration time']
    ]
    
    def prepare(self, *args, **opts):
        motors = args[0:-2:3]
        starts = args[1:-2:3]
        ends   = args[2:-2:3]
        nr_interv = args[-2]
        integ_time = args[-1]
        
        self._prepare(motors, starts, ends, nr_interv, integ_time, **opts)


class dscan(dNscan, Macro): 
    """motor scan relative to the starting position.
    dscan scans one motor, as specified by motor. If motor motor is at a
    position X before the scan begins, it will be scanned from X+start_pos
    to X+final_pos. The step size is (start_pos-final_pos)/nr_interv.
    The number of data points collected will be nr_interv+1. Count time is
    given by time which if positive, specifies seconds and if negative,
    specifies monitor counts. """

    param_def = [
       ['motor',      Type.Moveable,   None, 'Moveable to move'],
       ['start_pos',  Type.Float,   None, 'Scan start position'],
       ['final_pos',  Type.Float,   None, 'Scan final position'],
       ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
       ['integ_time', Type.Float,   None, 'Integration time']
    ]

    def prepare(self, motor, start_pos, final_pos, nr_interv, integ_time,
                **opts):
        self._prepare([motor], [start_pos], [final_pos], nr_interv, integ_time, **opts)


class d2scan(dNscan,Macro): 
    """two-motor scan relative to the starting position.
    d2scan scans two motors, as specified by motor1 and motor2.
    Each motor moves the same number of intervals. If each motor is at a
    position X before the scan begins, it will be scanned from X+start_posN
    to X+final_posN (where N is one of 1,2).
    The step size for each motor is (start_pos-final_pos)/nr_interv.
    The number of data points collected will be nr_interv+1.
    Count time is given by time which if positive, specifies seconds and
    if negative, specifies monitor counts."""
    param_def = [
       ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
       ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
       ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
       ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
       ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
       ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
       ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
       ['integ_time', Type.Float,   None, 'Integration time']
    ]

    def prepare(self, motor1, start_pos1, final_pos1, motor2, start_pos2, final_pos2, nr_interv, integ_time,
                **opts):
        self._prepare([motor1,motor2], [start_pos1,start_pos2], [final_pos1,final_pos2], nr_interv, integ_time, **opts)


class d3scan(dNscan, Macro): 
    """three-motor scan .
    d3scan scans three motors, as specified by motor1, motor2 and motor3.
    Each motor moves the same number of intervals. If each motor is at a
    position X before the scan begins, it will be scanned from X+start_posN
    to X+final_posN (where N is one of 1,2,3)
    The step size for each motor is (start_pos-final_pos)/nr_interv.
    The number of data points collected will be nr_interv+1.
    Count time is given by time which if positive, specifies seconds and
    if negative, specifies monitor counts."""
    param_def = [
       ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
       ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
       ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
       ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
       ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
       ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
       ['motor3',      Type.Moveable,   None, 'Moveable 3 to move'],
       ['start_pos3',  Type.Float,   None, 'Scan start position 3'],
       ['final_pos3',  Type.Float,   None, 'Scan final position 3'],
       ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
       ['integ_time', Type.Float,   None, 'Integration time']
    ]

    def prepare(self, m1, s1, f1,  m2, s2, f2, m3, s3, f3, nr_interv, integ_time,
                **opts):
        self._prepare([m1,m2,m3], [s1,s2,s3], [f1,f2,f3], nr_interv, integ_time, **opts)
        

class d4scan(dNscan, Macro): 
    """four-motor scan relative to the starting positions
    a4scan scans four motors, as specified by motor1, motor2, motor3 and motor4.
    Each motor moves the same number of intervals. If each motor is at a
    position X before the scan begins, it will be scanned from X+start_posN
    to X+final_posN (where N is one of 1,2,3,4).
    The step size for each motor is (start_pos-final_pos)/nr_interv.
    The number of data points collected will be nr_interv+1.
    Count time is given by time which if positive, specifies seconds and
    if negative, specifies monitor counts.
    Upon termination, the motors are returned to their starting positions.
    """
    param_def = [
       ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
       ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
       ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
       ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
       ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
       ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
       ['motor3',      Type.Moveable,   None, 'Moveable 3 to move'],
       ['start_pos3',  Type.Float,   None, 'Scan start position 3'],
       ['final_pos3',  Type.Float,   None, 'Scan final position 3'],
       ['motor4',      Type.Moveable,   None, 'Moveable 3 to move'],
       ['start_pos4',  Type.Float,   None, 'Scan start position 3'],
       ['final_pos4',  Type.Float,   None, 'Scan final position 3'],
       ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
       ['integ_time', Type.Float,   None, 'Integration time']
    ]

    def prepare(self, m1, s1, f1, m2, s2, f2, m3, s3, f3, m4, s4, f4, nr_interv,
                integ_time, **opts):
        self._prepare([m1,m2,m3,m4], [s1,s2,s3,s4], [f1,f2,f3,f4], nr_interv, integ_time, **opts)
        

class mesh(Macro,Hookable): 
    """2d grid scan  .
    The mesh scan traces out a grid using motor1 and motor2.
    The first motor scans from m1_start_pos to m1_final_pos using the specified
    number of intervals. The second motor similarly scans from m2_start_pos
    to m2_final_pos. Each point is counted for for integ_time seconds
    (or monitor counts, if integ_time is negative).
    The scan of motor1 is done at each point scanned by motor2. That is, the
    first motor scan is nested within the second motor scan.
    """
    
    hints = { 'scan' : 'mesh', 'allowsHooks': ('pre-scan', 'pre-move', 'post-move', 'pre-acq', 'post-acq', 'post-step', 'post-scan') }
    env = ('ActiveMntGrp',)
    
    param_def = [
       ['motor1',      Type.Moveable,   None, 'First motor to move'],
       ['m1_start_pos',Type.Float,   None, 'Scan start position for first motor'],
       ['m1_final_pos',Type.Float,   None, 'Scan final position for first motor'],
       ['m1_nr_interv',Type.Integer, None, 'Number of scan intervals'],
       ['motor2',      Type.Moveable,   None, 'Second motor to move'],
       ['m2_start_pos',Type.Float,   None, 'Scan start position for second motor'],
       ['m2_final_pos',Type.Float,   None, 'Scan final position for second motor'],
       ['m2_nr_interv',Type.Integer, None, 'Number of scan intervals'],
       ['integ_time',  Type.Float,   None, 'Integration time'],
       ['bidirectional',   Type.Boolean, False, 'Save time by scanning s-shaped']
    ]

    def prepare(self, m1, m1_start_pos, m1_final_pos, m1_nr_interv,
                m2, m2_start_pos, m2_final_pos, m2_nr_interv, integ_time,
                bidirectional, **opts):
        self.motors=[m1,m2]
        self.starts = numpy.array([m1_start_pos,m2_start_pos],dtype='d')
        self.finals = numpy.array([m1_final_pos,m2_final_pos],dtype='d')
        self.nr_intervs= numpy.array([m1_nr_interv, m2_nr_interv],dtype='i')
        self.integ_time = integ_time
        self.bidirectional_mode = bidirectional
        
        self.name=opts.get('name','mesh')
        
        generator=self._generator
        moveables=self.motors
        env=opts.get('env',{})
        constrains=[getCallable(cns) for cns in opts.get('constrains',[UNCONSTRAINED])]
    
        #Hooks are not always set at this point. We will call getHooks later on in the scan_loop
        #self.pre_scan_hooks = self.getHooks('pre-scan')
        #self.post_scan_hooks = self.getHooks('post-scan')

        self._gScan=SScan(self, generator, moveables, env, constrains)

    def _generator(self):
        step = {}
        step["integ_time"] =  self.integ_time
        step["pre-move-hooks"] = self.getHooks('pre-move')
        step["post-move-hooks"] = self.getHooks('post-move')
        step["pre-acq-hooks"] = self.getHooks('pre-acq')
        step["post-acq-hooks"] = self.getHooks('post-acq') +  self.getHooks('_NOHINTS_')
        step["post-step-hooks"] = self.getHooks('post-step')
        step["check_func"] = []
        m1start,m2start=self.starts
        m1end,m2end=self.finals
        points1,points2=self.nr_intervs+1
        point_no=1
        m1_space = numpy.linspace(m1start,m1end,points1)
        m1_space_inv = numpy.linspace(m1end,m1start,points1)
                                
        for i, m2pos in enumerate(numpy.linspace(m2start,m2end,points2)):
            space = m1_space
            if i % 2 != 0 and self.bidirectional_mode:
                space = m1_space_inv
            for m1pos in space:
                step["positions"] = numpy.array([m1pos,m2pos])
                step["point_id"]= point_no  #TODO: maybe another ID would be better? (e.g. "(A,B)")
                point_no+=1
                yield step
    
    def run(self,*args):
        for step in self._gScan.step_scan():
            yield step

    @property
    def data(self):
        return self._gScan.data

class fscan(Macro,Hookable):
    '''N-dimensional scan along user defined paths.
    The motion path for each motor is defined through the evaluation of a
    user-supplied function that is evaluated as a function of the independent
    variables.
    -independent variables are supplied through the indepvar string.
    The syntax for indepvar is "x=expresion1,y=expresion2,..."
    -If no indep vars need to be defined, write "!" or "*" or "None"
    -motion path for motor is generated by evaluating the corresponding
    function 'func'
    -Count time is given by integ_time. If integ_time is a scalar, then
    the same integ_time is used for all points. If it evaluates as an array
    (with same length as the paths), fscan will assign a different integration
    time to each acquisition point.
    -If integ_time is positive, it specifies seconds and if negative, specifies
    monitor counts.   

    IMPORTANT Notes:
    -no spaces are allowed in the indepvar string.
    -all funcs must evaluate to the same number of points
    
    EXAMPLE: fscan x=[1,3,5,7,9],y=arange(5) motor1 x**2 motor2 sqrt(y*x-3) 0.1
    '''
    
    hints = { 'scan' : 'fscan', 'allowsHooks': ('pre-scan', 'pre-move', 'post-move', 'pre-acq', 'post-acq', 'post-step', 'post-scan') }
    env = ('ActiveMntGrp',)
    
    param_def = [
       ['indepvars',  Type.String, None, 'Independent Variables'],
       ['motor_funcs',
        ParamRepeat(['motor', Type.Moveable, None, 'motor'],
                    ['func', Type.String, None, 'curve defining path']),
        None, 'List of motor and path curves'],
       ['integ_time', Type.String,   None, 'Integration time']
    ]
    
    def prepare(self, *args, **opts):
        if args[0].lower() in ["!", "*", "none", None]: indepvars={}
        else: indepvars=SafeEvaluator({'dict':dict}).eval('dict(%s)'%args[0]) #create a dict containing the indepvars
        self.motors=args[1:-1:2] #get motors
        sev=SafeEvaluator(indepvars) #create a safe evaluator whitelisting the indepvars
        self.funcstrings=args[2:-1:2]
        self.paths = map(sev.eval,self.funcstrings) #evaluate the functions
        self.integ_time=numpy.array(sev.eval(args[-1]), dtype='d')
        self.opts = opts
        if len(self.motors)==len(self.paths)>0:
            self.N=len(self.motors)
        else:
            raise ValueError('Moveable and func lists must be non-empty and same length')
        npoints=len(self.paths[0])
        try:
            #if everything is OK, the following lines should return a 2D array
            # n which each motor path is a row.
            #Typical failure is due to shape mismatch due to inconsistent input
            self.paths=numpy.array(self.paths,dtype='d')
            self.paths.reshape((self.N, npoints))
        except: #shape mismatch?
            #try to give a meaningful description of the error
            for p,fs in zip(self.paths,self.funcstrings):
                if len(p)!=npoints:
                    raise ValueError(
                        '"%s" and "%s" yield different number of points (%i vs %i)'
                        %(self.funcstrings[0],fs,npoints,len(p)))
            raise #the problem wasn't a shape mismatch
        self.nr_points=npoints
        
        if self.integ_time.size==1:
            self.integ_time=self.integ_time*numpy.ones(self.nr_points) #extend integ_time
        elif self.integ_time.size!=self.nr_points:
            raise ValueError('time_integ must either be a scalar or length=npoints (%i)'%self.nr_points)
                
        self.name=opts.get('name','fscan')
                
        generator=self._generator
        moveables=self.motors
        env=opts.get('env',{})
        constrains=[getCallable(cns) for cns in opts.get('constrains',[UNCONSTRAINED])]
        
        #Hooks are not always set at this point. We will call getHooks later on in the scan_loop
        #self.pre_scan_hooks = self.getHooks('pre-scan')
        #self.post_scan_hooks = self.getHooks('post-scan'
        
        self._gScan=SScan(self, generator, moveables, env, constrains)
        
    def _generator(self):
        step = {}
        step["pre-move-hooks"] = self.getHooks('pre-move')
        step["post-move-hooks"] = self.getHooks('post-move')
        step["pre-acq-hooks"] = self.getHooks('pre-acq')
        step["post-acq-hooks"] = self.getHooks('post-acq') + self.getHooks('_NOHINTS_')
        step["post-step-hooks"] = self.getHooks('post-step')
        
        step["check_func"] = []
        for i in xrange(self.nr_points):
            step["positions"] = self.paths[:,i]
            step["integ_time"] = self.integ_time[i]
            step["point_id"]= i
            yield step
    
    def run(self,*args):
        for step in self._gScan.step_scan():
            yield step

    @property
    def data(self):
        return self._gScan.data


class ascanh(aNscan, Macro): 
    """Do an absolute scan of the specified motor.
    ascan scans one motor, as specified by motor. The motor starts at the
    position given by start_pos and ends at the position given by final_pos.
    The step size is (start_pos-final_pos)/nr_interv. The number of data points collected
    will be nr_interv+1. Count time is given by time which if positive,
    specifies seconds and if negative, specifies monitor counts. """

    param_def = [
       ['motor',      Type.Moveable,   None, 'Moveable to move'],
       ['start_pos',  Type.Float,   None, 'Scan start position'],
       ['final_pos',  Type.Float,   None, 'Scan final position'],
       ['nr_interv',  Type.Integer, None, 'Number of scan intervals'],
       ['integ_time', Type.Float,   None, 'Integration time']
    ]

    def prepare(self, motor, start_pos, final_pos, nr_interv, integ_time,
                **opts):
        self._prepare([motor], [start_pos], [final_pos], nr_interv, integ_time,
                      mode=HybridMode, **opts)


class scanhist(Macro):
    """Shows scan history information. Give optional parameter scan number to
    display details about a specific scan"""
    
    param_def = [
       ['scan number',  Type.Integer, -1,
        'scan number. [default=-1 meaning show all scans]'],
    ]
    
    def run(self, scan_number):
        try:
            hist = self.getEnv("ScanHistory")
        except UnknownEnv:
            print "No scan recorded in history"
            return
        if scan_number < 0:
            self.show_all(hist)
        else:
            self.show_one(hist, scan_number)
    
    def show_one(self, hist, scan_number):
        item = None
        for h in hist:
            if h['serialno'] == scan_number:
                item = h
                break
        if item is None:
            self.warning("Could not find scan number %s", scan_number)
            return
        
        serialno, title = h['serialno'], h['title']
        start = datetime.datetime.fromtimestamp(h['startts'])
        end = datetime.datetime.fromtimestamp(h['endts'])
        total_time = end - start
        start, end, total_time = start.ctime(), end.ctime(), str(total_time)
        scan_dir, scan_file = h['ScanDir'], h['ScanFile']
        deadtime = '%.1f%%' % h['deadtime']
        
        user = h['user']
        store = "Not stored!"
        if scan_dir is not None and scan_file is not None:
            if type(scan_file) is str:
                store = os.path.join(scan_dir, scan_file)
            else:
                store = scan_dir + os.path.sep + str(scan_file)
        
        channels = ", ".join(h['channels'])
        cols = ["#", "Title", "Start time", "End time", "Took", "Dead time",
                "User", "Stored", "Channels" ]
        data = [serialno, title, start, end, total_time, deadtime, user, store,
                channels]
        
        table = Table([data], row_head_str=cols, row_head_fmt='%*s',
                      elem_fmt=['%-*s'],
                      col_sep='  :  ')
        for line in table.genOutput():
            self.output(line)
    
    def show_all(self, hist):
        
        cols  = "#", "Title", "Start time", "End time", "Stored"
        width =  -1,      -1,           -1,         -1,       -1
        out = List(cols, max_col_width=width)
        today = datetime.datetime.today().date()
        for h in hist:
            start = datetime.datetime.fromtimestamp(h['startts'])
            if start.date() == today:
                start = start.time().strftime("%H:%M:%S")
            else:
                start = start.strftime("%Y-%m-%d %H:%M:%S")
            end = datetime.datetime.fromtimestamp(h['endts'])
            if end.date() == today:
                end = end.time().strftime("%H:%M:%S")
            else:
                end = end.strftime("%Y-%m-%d %H:%M:%S")
            scan_file = h['ScanFile']
            store = "Not stored!"
            if scan_file is not None:
                store = ", ".join(scan_file)
            row = h['serialno'], h['title'], start, end, store
            out.appendRow(row)
        for line in out.genOutput():
            self.output(line)
        

class ascanc(aNscan, Macro): 
    """Do an absolute continuous scan of the specified motor.
    ascanc scans one motor, as specified by motor."""

    param_def = [
       ['motor',      Type.Moveable,   None, 'Moveable to move'],
       ['start_pos',  Type.Float,   None, 'Scan start position'],
       ['final_pos',  Type.Float,   None, 'Scan final position'],
       ['integ_time', Type.Float,   None, 'Integration time'],
       ['slow_down',  Type.Float, 1, 'global scan slow down factor (0, 1]'],
    ]

    def prepare(self, motor, start_pos, final_pos, integ_time, slow_down,
                **opts):
        self._prepare([motor], [start_pos], [final_pos], slow_down,
                      integ_time, mode=ContinuousMode, **opts)


class a2scanc(aNscan, Macro): 
    """two-motor continuous scan"""
    param_def = [
       ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
       ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
       ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
       ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
       ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
       ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
       ['integ_time', Type.Float,   None, 'Integration time'],
       ['slow_down',  Type.Float, 1, 'global scan slow down factor (0, 1]'],
    ]

    def prepare(self, motor1, start_pos1, final_pos1, motor2, start_pos2,
                final_pos2, integ_time, slow_down, **opts):
        self._prepare([motor1, motor2], [start_pos1, start_pos2],
                      [final_pos1, final_pos2], slow_down, integ_time,
                      mode=ContinuousMode, **opts)


class a3scanc(aNscan, Macro): 
    """three-motor continuous scan"""
    param_def = [
       ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
       ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
       ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
       ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
       ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
       ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
       ['motor3',      Type.Moveable,   None, 'Moveable 3 to move'],
       ['start_pos3',  Type.Float,   None, 'Scan start position 3'],
       ['final_pos3',  Type.Float,   None, 'Scan final position 3'],
       ['integ_time', Type.Float,   None, 'Integration time'],
       ['slow_down',  Type.Float, 1, 'global scan slow down factor (0, 1]'],
    ]

    def prepare(self, m1, s1, f1,  m2, s2, f2, m3, s3, f3, integ_time,
                slow_down, **opts):
        self._prepare([m1,m2,m3], [s1,s2,s3], [f1,f2,f3], slow_down,
                      integ_time, mode=ContinuousMode, **opts)

class a4scanc(aNscan, Macro): 
    """four-motor continuous scan"""
    param_def = [
       ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
       ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
       ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
       ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
       ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
       ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
       ['motor3',      Type.Moveable,   None, 'Moveable 3 to move'],
       ['start_pos3',  Type.Float,   None, 'Scan start position 3'],
       ['final_pos3',  Type.Float,   None, 'Scan final position 3'],
       ['motor4',      Type.Moveable,   None, 'Moveable 3 to move'],
       ['start_pos4',  Type.Float,   None, 'Scan start position 3'],
       ['final_pos4',  Type.Float,   None, 'Scan final position 3'],
       ['integ_time', Type.Float,   None, 'Integration time'],
       ['slow_down',  Type.Float, 1, 'global scan slow down factor (0, 1]'],
    ]

    def prepare(self, m1, s1, f1, m2, s2, f2, m3, s3, f3, m4, s4, f4,
                integ_time, slow_down, **opts):
        self._prepare([m1,m2,m3,m4], [s1,s2,s3,s4], [f1,f2,f3,f4], slow_down,
                      integ_time, mode=ContinuousMode, **opts)


class dNscanc(dNscan):
    
    def do_restore(self):
        # set velocities to maximum and then move to initial positions
        for moveable in self.motors:
            self._gScan.set_max_top_velocity(moveable)
        dNscan.do_restore(self)


class dscanc(dNscanc, Macro): 
    """continuous motor scan relative to the starting position."""

    param_def = [
       ['motor',      Type.Moveable,   None, 'Moveable to move'],
       ['start_pos',  Type.Float,   None, 'Scan start position'],
       ['final_pos',  Type.Float,   None, 'Scan final position'],
       ['integ_time', Type.Float,   None, 'Integration time'],
       ['slow_down',  Type.Float, 1, 'global scan slow down factor (0, 1]'],
    ]

    def prepare(self, motor, start_pos, final_pos, integ_time, slow_down,
                **opts):
        self._prepare([motor], [start_pos], [final_pos], slow_down, integ_time,
                       mode=ContinuousMode, **opts)


class d2scanc(dNscanc,Macro): 
    """continuous two-motor scan relative to the starting positions"""
    param_def = [
       ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
       ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
       ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
       ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
       ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
       ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
       ['integ_time', Type.Float,   None, 'Integration time'],
       ['slow_down',  Type.Float, 1, 'global scan slow down factor (0, 1]'],
    ]

    def prepare(self, motor1, start_pos1, final_pos1, motor2, start_pos2,
                final_pos2, integ_time, slow_down, **opts):
        self._prepare([motor1,motor2], [start_pos1,start_pos2],
                      [final_pos1,final_pos2], slow_down, integ_time,
                       mode=ContinuousMode, **opts)


class d3scanc(dNscanc, Macro): 
    """continuous three-motor scan"""
    param_def = [
       ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
       ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
       ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
       ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
       ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
       ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
       ['motor3',      Type.Moveable,   None, 'Moveable 3 to move'],
       ['start_pos3',  Type.Float,   None, 'Scan start position 3'],
       ['final_pos3',  Type.Float,   None, 'Scan final position 3'],
       ['integ_time', Type.Float,   None, 'Integration time'],
       ['slow_down',  Type.Float, 1, 'global scan slow down factor (0, 1]'],
    ]

    def prepare(self, m1, s1, f1,  m2, s2, f2, m3, s3, f3, integ_time, slow_down,
                **opts):
        self._prepare([m1,m2,m3], [s1,s2,s3], [f1,f2,f3], slow_down, integ_time,
                       mode=ContinuousMode, **opts)
        

class d4scanc(dNscanc, Macro): 
    """continuous four-motor scan relative to the starting positions"""
    param_def = [
       ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
       ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
       ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
       ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
       ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
       ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
       ['motor3',      Type.Moveable,   None, 'Moveable 3 to move'],
       ['start_pos3',  Type.Float,   None, 'Scan start position 3'],
       ['final_pos3',  Type.Float,   None, 'Scan final position 3'],
       ['motor4',      Type.Moveable,   None, 'Moveable 3 to move'],
       ['start_pos4',  Type.Float,   None, 'Scan start position 3'],
       ['final_pos4',  Type.Float,   None, 'Scan final position 3'],
       ['integ_time', Type.Float,   None, 'Integration time'],
       ['slow_down',  Type.Float, 1, 'global scan slow down factor (0, 1]'],
    ]

    def prepare(self, m1, s1, f1, m2, s2, f2, m3, s3, f3, m4, s4, f4, integ_time,
                slow_down, **opts):
        self._prepare([m1,m2,m3,m4], [s1,s2,s3,s4], [f1,f2,f3,f4], slow_down,
                      integ_time, mode=ContinuousMode, **opts)


class meshc(Macro,Hookable): 
    """2d grid scan. scans continuous"""
    
    hints = { 'scan' : 'mesh', 'allowsHooks': ('pre-scan', 'pre-move', 'post-move', 'pre-acq', 'post-acq', 'post-step', 'post-scan') }
    env = ('ActiveMntGrp',)
    
    param_def = [
       ['motor1',        Type.Moveable, None, 'First motor to move'],
       ['m1_start_pos',  Type.Float,    None, 'Scan start position for first motor'],
       ['m1_final_pos',  Type.Float,    None, 'Scan final position for first motor'],
       ['slow_down',     Type.Float,    None, 'global scan slow down factor (0, 1]'],
       ['motor2',        Type.Moveable, None, 'Second motor to move'],
       ['m2_start_pos',  Type.Float,    None, 'Scan start position for second motor'],
       ['m2_final_pos',  Type.Float,    None, 'Scan final position for second motor'],
       ['m2_nr_interv',  Type.Integer,  None, 'Number of scan intervals'],
       ['integ_time',    Type.Float,    None, 'Integration time'],
       ['bidirectional', Type.Boolean,  False, 'Save time by scanning s-shaped']
    ]

    def prepare(self, m1, m1_start_pos, m1_final_pos, slow_down,
                m2, m2_start_pos, m2_final_pos, m2_nr_interv, integ_time,
                bidirectional, **opts):
        self.motors=[m1,m2]
        self.slow_down = slow_down
        self.starts = numpy.array([m1_start_pos,m2_start_pos],dtype='d')
        self.finals = numpy.array([m1_final_pos,m2_final_pos],dtype='d')
        self.m2_nr_interv = m2_nr_interv
        self.integ_time = integ_time
        self.bidirectional_mode = bidirectional
        self.nr_waypoints = m2_nr_interv + 1
        
        self.name=opts.get('name','meshc')
        
        moveables=self.motors
        env=opts.get('env',{})
        constrains=[getCallable(cns) for cns in opts.get('constrains',[UNCONSTRAINED])]
        extrainfodesc = opts.get('extrainfodesc',[])
    
        #Hooks are not always set at this point. We will call getHooks later on in the scan_loop
        #self.pre_scan_hooks = self.getHooks('pre-scan')
        #self.post_scan_hooks = self.getHooks('post-scan'

        self._gScan = CScan(self, self._waypoint_generator, self._period_generator, moveables, env, constrains, extrainfodesc)
        self._gScan.frozen_motors = [m2]
        
    def _waypoint_generator(self):
        step = {}
        step["pre-move-hooks"] = self.getHooks('pre-move')
        step["post-move-hooks"] = self.getHooks('post-move')
        step["check_func"] = []
        step["slow_down"] = self.slow_down
        points2=self.m2_nr_interv+1
        m1start,m2start=self.starts
        m1end,m2end=self.finals
        point_no=1
        for i, m2pos in enumerate(numpy.linspace(m2start,m2end,points2)):
            start, end = m1start, m1end
            if i % 2 != 0 and self.bidirectional_mode:
                start, end = m1end, m1start
            step["start_positions"] = numpy.array([start, m2pos])
            step["positions"] = numpy.array([end, m2pos])
            step["point_id"]= point_no
            point_no+=1
            yield step
    
    def _period_generator(self):
        step = {}
        step["integ_time"] =  self.integ_time
        step["pre-acq-hooks"] = self.getHooks('pre-acq')
        step["post-acq-hooks"] = self.getHooks('post-acq')+self.getHooks('_NOHINTS_')
        step["post-step-hooks"] = self.getHooks('post-step')
        step["check_func"] = []
        step['extrainfo'] = {}
        point_no = 0
        while(True):
            point_no += 1
            step["point_id"] = point_no
            yield step

    def run(self,*args):
        for step in self._gScan.step_scan():
            yield step

    def getTimeEstimation(self):
        return self._gScan.waypoint_estimation()

    def getIntervalEstimation(self):
        return self.nr_waypoints

    @property
    def data(self):
        return self._gScan.data
    
class ascanct(aNscan, Macro):
    '''Continuous scan controlled by hardware trigger signals. A sequence of 
    trigger pulses is programmed by time. The scan active time is calculated from
    nr_of_points * point time. It corresponds to the time while all the involved 
    in the scan moveables are at the constant velocity. Experimental channels are
    configured to acquire during acquistion time calculated from acq_time [%] of  
    the point_time. 
    Temporary solution used to configure trigger device (pulse train generator): 
    "TriggerDevices" evironment variable must be set to Ni660X device name 
    or any other Tango device name implementing:
    +) following attributes: 
    - InitialDelayTime [s] - delay time from calling Start to generating first pulse 
    - HighTime [s] - time interval while signal will maintain its high state
    - LowTime [s] - time interval while signal will maintain its low state
    - SampPerChan - nr of pulses to be generated
    - IdleState - state (high or low) which signal will take after the Start command
      and which will maintain during the InitialDelayTime.  
    +) following commands:
    - Start 
    - Stop)'''

    hints = {'scan' : 'ascanct', 'allowsHooks': ('pre-configuration', 
                                                 'post-configuration', 
                                                 'pre-start', 
                                                 'pre-cleanup', 
                                                 'post-cleanup') }

    param_def = [['motor', Type.Moveable, None, 'Moveable name'],
                 ['start_pos', Type.Float, None, 'Starting position'],
                 ['end_pos', Type.Float, None, 'Ending pos value'],
                 ['nr_of_points', Type.Integer, None, 'Nr of scan points'],
                 ['point_time', Type.Float, None, 'Time interval reserved for ' + 
                                                   'each scan point [s].'],
                 ['acq_time', Type.Float, 99, 'Acquisition time per scan point. ' +
                      'Expressed in percentage of point_time. Default: 99 [%]'],
                 ['samp_freq', Type.Float, -1, 'Sampling frequency. ' + 
                                        'Default: -1 (means maximum possible)']]

        
    def prepare(self, motor, start_pos, end_pos, nr_of_points, 
                point_time, acq_time, samp_freq, **opts):
        self._prepare([motor], [start_pos], [end_pos], nr_of_points,
                      point_time, mode=ContinuousHwTimeMode, **opts)
        self.acq_time = acq_time
        self.samp_freq = samp_freq


class a2scanct(aNscan, Macro):
    '''Continuous scan controlled by hardware trigger signals. A sequence of 
    trigger pulses is programmed by time. The scan active time is calculated from
    nr_of_points * point time. It corresponds to the time while all the involved 
    in the scan moveables are at the constant velocity. Experimental channels are
    configured to acquire during acquistion time calculated from acq_time [%] of  
    the point_time. 
    Temporary solution used to configure trigger device (pulse train generator): 
    "TriggerDevices" evironment variable must be set to Ni660X device name 
    or any other Tango device name implementing:
    +) following attributes: 
    - InitialDelayTime [s] - delay time from calling Start to generating first pulse 
    - HighTime [s] - time interval while signal will maintain its high state
    - LowTime [s] - time interval while signal will maintain its low state
    - SampPerChan - nr of pulses to be generated
    - IdleState - state (high or low) which signal will take after the Start command
      and which will maintain during the InitialDelayTime.  
    +) following commands:
    - Start 
    - Stop)'''

    hints = {'scan' : 'a2scanct', 'allowsHooks': ('pre-configuration', 
                                                  'post-configuration',
                                                  'pre-start',
                                                  'pre-cleanup',
                                                  'post-cleanup') }

    param_def = [
           ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
           ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
           ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
           ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
           ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
           ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
           ["nr_of_points", Type.Integer, None, "Nr of scan points"],
           ['point_time', Type.Float,   None, 'Time interval reserved for ' + 
                                              'each scan point [s].'],
           ["acq_time", Type.Float, 99, 'Acquisition time per scan point. ' + 
                      'Expressed in percentage of point_time. Default: 99 [%]'],
           ["samp_freq", Type.Float, -1, 'Sampling frequency. ' + 
                                       'Default: -1 (means maximum possible)']]
           
    def prepare(self, m1, s1, f1, m2, s2, f2, nr_of_points, 
                point_time, acq_time, samp_freq, **opts):
        self._prepare([m1, m2], [s1, s2], [f1, f2], nr_of_points,
                      point_time, mode=ContinuousHwTimeMode, **opts)
        self.acq_time = acq_time
        self.samp_freq = samp_freq
        
        
        
class a3scanct(aNscan, Macro):
    '''Continuous scan controlled by hardware trigger signals. A sequence of 
    trigger pulses is programmed by time. The scan active time is calculated from
    nr_of_points * point time. It corresponds to the time while all the involved 
    in the scan moveables are at the constant velocity. Experimental channels are
    configured to acquire during acquistion time calculated from acq_time [%] of  
    the point_time. 
    Temporary solution used to configure trigger device (pulse train generator): 
    "TriggerDevices" evironment variable must be set to Ni660X device name 
    or any other Tango device name implementing:
    +) following attributes: 
    - InitialDelayTime [s] - delay time from calling Start to generating first pulse 
    - HighTime [s] - time interval while signal will maintain its high state
    - LowTime [s] - time interval while signal will maintain its low state
    - SampPerChan - nr of pulses to be generated
    - IdleState - state (high or low) which signal will take after the Start command
      and which will maintain during the InitialDelayTime.  
    +) following commands:
    - Start 
    - Stop)'''
    hints = {'scan' : 'a2scanct', 'allowsHooks': ('pre-configuration', 
                                                  'post-configuration',
                                                  'pre-start',
                                                  'pre-cleanup',
                                                  'post-cleanup') }

    param_def = [
           ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
           ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
           ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
           ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
           ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
           ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
           ['motor3',      Type.Moveable,   None, 'Moveable 3 to move'],
           ['start_pos3',  Type.Float,   None, 'Scan start position 3'],
           ['final_pos3',  Type.Float,   None, 'Scan final position 3'],
           ["nr_of_points", Type.Integer, None, "Nr of scan points"],
           ['point_time', Type.Float,   None, 'Time interval reserved for ' + 
                                              'each scan point [s].'],
           ["acq_time", Type.Float, 99, 'Acquisition time per scan point. ' + 
                      'Expressed in percentage of point_time. Default: 99 [%]'],
           ["samp_freq", Type.Float, -1, 'Sampling frequency. ' + 
                                       'Default: -1 (means maximum possible)']]
           
    def prepare(self, m1, s1, f1, m2, s2, f2, m3, s3, f3, nr_of_points, 
                point_time, acq_time, samp_freq, **opts):
        self._prepare([m1, m2, m3], [s1, s2, s3], [f1, f2, f3], nr_of_points,
                      point_time, mode=ContinuousHwTimeMode, **opts)
        self.acq_time = acq_time
        self.samp_freq = samp_freq
        
class a4scanct(aNscan, Macro):
    '''Continuous scan controlled by hardware trigger signals. A sequence of 
    trigger pulses is programmed by time. The scan active time is calculated from
    nr_of_points * point time. It corresponds to the time while all the involved 
    in the scan moveables are at the constant velocity. Experimental channels are
    configured to acquire during acquistion time calculated from acq_time [%] of  
    the point_time. 
    Temporary solution used to configure trigger device (pulse train generator): 
    "TriggerDevices" evironment variable must be set to Ni660X device name 
    or any other Tango device name implementing:
    +) following attributes: 
    - InitialDelayTime [s] - delay time from calling Start to generating first pulse 
    - HighTime [s] - time interval while signal will maintain its high state
    - LowTime [s] - time interval while signal will maintain its low state
    - SampPerChan - nr of pulses to be generated
    - IdleState - state (high or low) which signal will take after the Start command
      and which will maintain during the InitialDelayTime.  
    +) following commands:
    - Start 
    - Stop)'''

    hints = {'scan' : 'a2scanct', 'allowsHooks': ('pre-configuration', 
                                                  'post-configuration',
                                                  'pre-start',
                                                  'pre-cleanup',
                                                  'post-cleanup') }

    param_def = [
           ['motor1',      Type.Moveable,   None, 'Moveable 1 to move'],
           ['start_pos1',  Type.Float,   None, 'Scan start position 1'],
           ['final_pos1',  Type.Float,   None, 'Scan final position 1'],
           ['motor2',      Type.Moveable,   None, 'Moveable 2 to move'],
           ['start_pos2',  Type.Float,   None, 'Scan start position 2'],
           ['final_pos2',  Type.Float,   None, 'Scan final position 2'],
           ['motor3',      Type.Moveable,   None, 'Moveable 3 to move'],
           ['start_pos3',  Type.Float,   None, 'Scan start position 3'],
           ['final_pos3',  Type.Float,   None, 'Scan final position 3'],
           ['motor4',      Type.Moveable,   None, 'Moveable 4 to move'],
           ['start_pos4',  Type.Float,   None, 'Scan start position 4'],
           ['final_pos4',  Type.Float,   None, 'Scan final position 4'],
           ["nr_of_points", Type.Integer, None, "Nr of scan points"],
           ['point_time', Type.Float,   None, 'Time interval reserved for ' + 
                                              'each scan point [s].'],
           ["acq_time", Type.Float, 99, 'Acquisition time per scan point. ' + 
                      'Expressed in percentage of point_time. Default: 99 [%]'],
           ["samp_freq", Type.Float, -1, 'Sampling frequency. ' + 
                                       'Default: -1 (means maximum possible)']]
   
           
    def prepare(self, m1, s1, f1, m2, s2, f2, m3, s3, f3, m4, s4, f4, 
                nr_of_points, point_time, acq_time, samp_freq, **opts):
        self._prepare([m1, m2, m3, m4], [s1, s2, s3, s4], [f1, f2, f3, f4],
                      nr_of_points, point_time, mode=ContinuousHwTimeMode,
                      **opts)
        self.acq_time = acq_time
        self.samp_freq = samp_freq
        



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

""" """

__all__ = ["MotorGroup", "MotorGroupClass"]

__docformat__ = 'restructuredtext'

import sys
import time

from PyTango import Except, DevState, DevDouble, AttrQuality, READ_WRITE, \
    SPECTRUM

from taurus.core.util.log import DebugIt

from sardana import State, SardanaServer
from sardana.sardanaattribute import SardanaAttribute
from sardana.pool.poolexception import PoolException
from sardana.tango.core.util import exception_str, throw_sardana_exception
from sardana.tango.pool.PoolDevice import PoolGroupDevice, PoolGroupDeviceClass


class MotorGroup(PoolGroupDevice):

    def __init__(self, dclass, name):
        self.in_write_position = False
        PoolGroupDevice.__init__(self, dclass, name)

    def init(self, name):
        PoolGroupDevice.init(self, name)

    def _is_allowed(self, req_type):
        return PoolGroupDevice._is_allowed(self, req_type)

    def get_motor_group(self):
        return self.element

    def set_motor_group(self, motor_group):
        self.element = motor_group

    motor_group = property(get_motor_group, set_motor_group)

    @DebugIt()
    def delete_device(self):
        PoolGroupDevice.delete_device(self)
        motor_group = self.motor_group
        if motor_group is not None:
            motor_group.remove_listener(self.on_motor_group_changed)

    @DebugIt()
    def init_device(self):
        PoolGroupDevice.init_device(self)
        detect_evts = "position",
        non_detect_evts = "elementlist",
        self.set_change_events(detect_evts, non_detect_evts)
        self.Elements = map(int, self.Elements)
        motor_group = self.motor_group
        if motor_group is None:
            full_name = self.get_full_name()
            name = self.alias or full_name
            self.motor_group = motor_group = \
                self.pool.create_motor_group(name=name, id=self.Id,
                    full_name=full_name, user_elements=self.Elements)
        motor_group.add_listener(self.on_motor_group_changed)
        self.set_state(DevState.ON)

    def on_motor_group_changed(self, event_source, event_type, event_value):
        try:
            self._on_motor_group_changed(event_source, event_type, event_value)
        except:
            msg = 'Error occurred "on_motor_group_changed(%s.%s): %s"'
            exc_info = sys.exc_info()
            self.error(msg, self.motor_group.name, event_type.name,
                       exception_str(*exc_info[:2]))
            self.debug("Details", exc_info=exc_info)

    def _on_motor_group_changed(self, event_source, event_type, event_value):
        # during server startup and shutdown avoid processing element
        # creation events
        if SardanaServer.server_state != State.Running:
            return

        timestamp = time.time()
        name = event_type.name.lower()

        attr = self.get_attribute_by_name(name)
        quality = AttrQuality.ATTR_VALID
        priority = event_type.priority
        value, w_value, error = None, None, None

        if name == "state":
            value = self.calculate_tango_state(event_value)
        elif name == "status":
            value = self.calculate_tango_status(event_value)
        else:
            if isinstance(event_value, SardanaAttribute):
                if event_value.error:
                    error = Except.to_dev_failed(*event_value.exc_info)
                else:
                    value = event_value.value
                timestamp = event_value.timestamp
            else:
                value = event_value

            state = self.motor_group.get_state(propagate=0)

            if name == "position":
                w_value = event_value.w_value
                if state == State.Moving:
                    quality = AttrQuality.ATTR_CHANGING

        self.set_attribute(attr, value=value, w_value=w_value,
                           timestamp=timestamp, quality=quality,
                           priority=priority, error=error, synch=False)

    def always_executed_hook(self):
        pass
        #state = to_tango_state(self.motor_group.get_state(cache=False))

    def read_attr_hardware(self, data):
        pass

    def _to_motor_positions(self, pos):
        positions = []
        for elem in self.motor_group.get_user_elements():
            position = pos[elem]
            if position.in_error():
                Except.throw_python_exception(*position.exc_info)
            positions.append(position.value)
        return positions

    def _to_motor_write_positions(self, pos):
        w_positions = []
        for elem in self.motor_group.get_user_elements():
            position = pos[elem]
            if position.in_error():
                Except.throw_python_exception(*position.exc_info)
            w_positions.append(position.w_value)
        return w_positions

    def read_Position(self, attr):
        # if motors are moving their position is already being updated with a
        # high frequency so don't bother overloading and just get the cached
        # values
        motor_group = self.motor_group
        use_cache = motor_group.is_in_operation() and not self.Force_HW_Read
        position = motor_group.get_position(cache=use_cache, propagate=0)
        if position.error:
            Except.throw_python_exception(*position.exc_info)
        state = motor_group.get_state(cache=use_cache, propagate=0)
        quality = None
        if state == State.Moving:
            quality = AttrQuality.ATTR_CHANGING
        self.set_attribute(attr, value=position.value, w_value=position.w_value,
                           quality=quality, priority=0,
                           timestamp=position.timestamp)

    def write_Position(self, attr):
        self.in_write_position = True
        try:
            position = attr.get_write_value()
            self.debug("write_Position(%s)", position)
            try:
                self.wait_for_operation()
            except:
                raise Exception("Cannot move: already in motion")
            try:
                self.motor_group.position = position
            except PoolException, pe:
                throw_sardana_exception(pe)
        finally:
            self.in_write_position = False

    is_Position_allowed = _is_allowed


class MotorGroupClass(PoolGroupDeviceClass):

    #    Class Properties
    class_property_list = {
    }

    #    Device Properties
    device_property_list = {
    }
    device_property_list.update(PoolGroupDeviceClass.device_property_list)

    #    Command definitions
    cmd_list = {
    }
    cmd_list.update(PoolGroupDeviceClass.cmd_list)

    #    Attribute definitions
    attr_list = {
        'Position'     : [ [ DevDouble, SPECTRUM, READ_WRITE, 4096 ], ],
    }
    attr_list.update(PoolGroupDeviceClass.attr_list)

    def _get_class_properties(self):
        ret = PoolGroupDeviceClass._get_class_properties(self)
        ret['Description'] = "Motor group device class"
        ret['InheritedFrom'].insert(0, 'PoolGroupDevice')
        return ret



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module contains the most generic sardana constants and enumerations"""

from __future__ import absolute_import

__all__ = ["EpsilonError", "SardanaServer", "ServerRunMode", "State",
           "DataType", "DataFormat", "DataAccess", "DTYPE_MAP", "R_DTYPE_MAP",
           "DACCESS_MAP",
           "from_dtype_str", "from_access_str", "to_dtype_dformat",
           "to_daccess", "InvalidId", "InvalidAxis", "ElementType",
           "Interface", "Interfaces", "InterfacesExpanded",
           "TYPE_ELEMENTS", "TYPE_GROUP_ELEMENTS", "TYPE_MOVEABLE_ELEMENTS",
           "TYPE_PHYSICAL_ELEMENTS", "TYPE_ACQUIRABLE_ELEMENTS",
           "TYPE_EXP_CHANNEL_ELEMENTS", "TYPE_TIMERABLE_ELEMENTS",
           "TYPE_PSEUDO_ELEMENTS", "INTERFACES", "INTERFACES_EXPANDED",
           "ScalarNumberFilter"]

__docformat__ = 'restructuredtext'

import math

from taurus.core.util.enumeration import Enumeration

#: maximum difference between two floats so that they are considered equal
EpsilonError = 1E-16

#: sardana element state enumeration
State = Enumeration("State", (\
    "On",
    "Off",
    "Close",
    "Open",
    "Insert",
    "Extract",
    "Moving",
    "Standby",
    "Fault",
    "Init",
    "Running",
    "Alarm",
    "Disable",
    "Unknown",
    "Invalid"))


class _SardanaServer(object):
    """Class representing the current sardana server state"""

    def __init__(self):
        self.server_state = State.Invalid

    def __repr__(self):
        return "SardanaServer()"


#: the global object containing the SardanaServer information
SardanaServer = _SardanaServer()

#:
#: The sardana server run mode:
#:
#: - **SynchPure** : Pure synchronous: Start the server and run the server loop
#:   until it stops
#: - **SynchThread** : separate thread synchronous: start a thread running the
#:   server loop. Block until the server loop ends
#: - **SynchProcess** : separate process synchronous: start a sub-process
#:   running the server loop. Block until the server loop ends
#: - **AsynchThread** : separate thread asynchronous: start a thread running the
#:   server loop. Return immediately
#: - **ASynchProcess** : separate process asynchronous: start a sub-process
#:   running the server loop. Return immediately
ServerRunMode = Enumeration("ServerRunMode", \
                            ("SynchPure", "SynchThread", "SynchProcess", \
                             "AsynchThread", "AsynchProcess"))

#: sardana data types (used by device pool controllers)
DataType = Enumeration("DataType", (\
    "Integer",
    "Double",
    "String",
    "Boolean",
    "Encoded",
    "Invalid"))

#: sardana data format enumeration (used by device pool controllers)
DataFormat = Enumeration("DataFormat", (\
    "Scalar",
    "OneD",
    "TwoD",
    "Invalid"))

#: sardana data access (used by device pool controllers)
DataAccess = Enumeration("DataAccess", (\
    "ReadOnly",
    "ReadWrite",
    "Invalid"))

#: dictionary dict<data type, :class:`sardana.DataType`>
DTYPE_MAP = {
    'int'            : DataType.Integer,
    'integer'        : DataType.Integer,
    int              : DataType.Integer,
    long             : DataType.Integer,
    'long'           : DataType.Integer,
    DataType.Integer : DataType.Integer,
    'float'          : DataType.Double,
    'double'         : DataType.Double,
    float            : DataType.Double,
    DataType.Double  : DataType.Double,
    'str'            : DataType.String,
    'string'         : DataType.String,
    str              : DataType.String,
    DataType.String  : DataType.String,
    'bool'           : DataType.Boolean,
    'boolean'        : DataType.Boolean,
    bool             : DataType.Boolean,
    DataType.Boolean : DataType.Boolean,
}

#: dictionary dict<data type, :class:`sardana.DataType`>
R_DTYPE_MAP = {
    'int'            : int,
    'integer'        : int,
    int              : int,
    long             : int,
    'long'           : int,
    DataType.Integer : int,
    'float'          : float,
    'double'         : float,
    float            : float,
    DataType.Double  : float,
    'str'            : str,
    'string'         : str,
    str              : str,
    DataType.String  : str,
    'bool'           : bool,
    'boolean'        : bool,
    bool             : bool,
    DataType.Boolean : bool,
}

#DTYPE_MAP.setdefault(DataType.Invalid)

#: dictionary dict<access type, :class:`sardana.DataAccess`>
DACCESS_MAP = {
    'read'               : DataAccess.ReadOnly,
    DataAccess.ReadOnly  : DataAccess.ReadOnly,
    'readwrite'          : DataAccess.ReadWrite,
    'read_write'         : DataAccess.ReadWrite,
    DataAccess.ReadWrite : DataAccess.ReadWrite,
}
#DACCESS_MAP.setdefault(DataAccess.Invalid)

def from_dtype_str(dtype):
    """Transforms the given dtype parameter (string/:obj:`DataType` or None)
    into a tuple of two elements (str, :obj:`DataFormat`) where the first
    element is a string with a simplified data type.
    
        - If None is given, it returns
          ('float', :obj:`DataFormat.Scalar`)
        - If :obj:`DataType` is given, it returns
          (:obj:`DataType`, :obj:`DataFormat.Scalar`)
          
    :param dtype: the data type to be transformed
    :type dtype: str or None or :obj:`DataType`
    :return: a tuple <str, :obj:`DataFormat`> for the given dtype
    :rtype: tuple<str, :obj:`DataFormat`>"""
    dformat = DataFormat.Scalar
    if dtype is None:
        dtype = 'float'
    elif isinstance(dtype, (str, unicode)):
        dtype = dtype.lower()
        if dtype.startswith("pytango."):
            dtype = dtype[len("pytango."):]
        if dtype.startswith("dev"):
            dtype = dtype[len("dev"):]
        if dtype.startswith("var"):
            dtype = dtype[len("var"):]
        if dtype.endswith("array"):
            dtype = dtype[:dtype.index("array")]
            dformat = DataFormat.OneD
    return dtype, dformat

def from_access_str(access):
    """Transforms the given access parameter (string or :obj:`DataAccess`) into
    a simplified data access string.
    
    :param dtype: the access to be transformed
    :type dtype: str
    :return: a simple string for the given access
    :rtype: str"""
    if isinstance(access, (str, unicode)):
        access = access.lower()
        if access.startswith("pytango."):
            access = access[len("pytango."):]
    return access

def to_dtype_dformat(data):
    """Transforms the given data parameter (string/ or sequence of string or
    sequence of sequence of string/:obj:`DataType`) into a tuple of two
    elements (:obj:`DataType`, :obj:`DataFormat`).
    
    :param data: the data information to be transformed
    :type data: str or seq<str> or seq<seq<str>>
    :return: a tuple <:obj:`DataType`, :obj:`DataFormat`> for the given data
    :rtype: tuple<:obj:`DataType`, :obj:`DataFormat`>
    """
    import operator
    dtype, dformat = data, DataFormat.Scalar
    if isinstance(data, (str, unicode)):
        dtype, dformat = from_dtype_str(data)
    elif operator.isSequenceType(data):
        dformat = DataFormat.OneD
        dtype = data[0]
        if type(dtype) == str:
            dtype, dformat2 = from_dtype_str(dtype)
            if dformat2 == DataFormat.OneD:
                dformat = DataFormat.TwoD
        elif operator.isSequenceType(dtype):
            dformat = DataFormat.TwoD
            dtype = dtype[0]
            if type(dtype) == str:
                dtype, _ = from_dtype_str(dtype)
    dtype = DTYPE_MAP.get(dtype, DataType.Invalid)
    return dtype, dformat

def to_daccess(daccess):
    """Transforms the given access parameter (string or None) into a
    :obj:`DataAccess`. If None is given returns :obj:`DataAccess.ReadWrite`
    
    :param dtype: the access to be transformed
    :type dtype: str
    :return: a :obj:`DataAccess` for the given access
    :rtype: :obj:`DataAccess`"""
    if daccess is None:
        daccess = DataAccess.ReadWrite
    elif isinstance(daccess , (str, unicode)):
        daccess = DACCESS_MAP.get(from_access_str(daccess), DataAccess.ReadWrite)
    return daccess

#: A constant representing  an invalid ID
InvalidId = 0

#: A constant representing an invalid axis
InvalidAxis = 0

#: An enumeration describing the all possible element types in sardana
ElementType = Enumeration("ElementType", (\
    "Pool",
    "Controller",
    "Motor",
    "CTExpChannel",
    "ZeroDExpChannel",
    "OneDExpChannel",
    "TwoDExpChannel",
    "ComChannel",
    "IORegister",
    "PseudoMotor",
    "PseudoCounter",
    "Constraint",
    "MotorGroup",
    "MeasurementGroup",
    "Instrument",
    "ControllerClass",
    "ControllerLibrary",
    "MacroServer",
    "Door",
    "MacroClass",
    "MacroLibrary",
    "MacroFunction",
    "External",
    "Meta",
    "ParameterType",
    "Unknown"))

ET = ElementType

#: a set containning all "controllable" element types.
#: Constant values belong to :class:`~sardana.sardanadefs.ElementType`
TYPE_ELEMENTS = set((ET.Motor, ET.CTExpChannel, ET.ZeroDExpChannel, \
    ET.OneDExpChannel, ET.TwoDExpChannel, \
    ET.ComChannel, ET.IORegister, ET.PseudoMotor, \
    ET.PseudoCounter, ET.Constraint))

#: a set containing all group element types.
#: Constant values belong to :class:`~sardana.sardanadefs.ElementType`
TYPE_GROUP_ELEMENTS = set((ET.MotorGroup, ET.MeasurementGroup))

#: a set containing the type of elements which are moveable.
#: Constant values belong to :class:`~sardana.sardanadefs.ElementType`
TYPE_MOVEABLE_ELEMENTS = set((ET.Motor, ET.PseudoMotor, ET.MotorGroup))

#: a set containing the possible types of physical elements.
#: Constant values belong to :class:`~sardana.sardanadefs.ElementType`
TYPE_PHYSICAL_ELEMENTS = set((ET.Motor, ET.CTExpChannel, ET.ZeroDExpChannel, \
    ET.OneDExpChannel, ET.TwoDExpChannel, \
    ET.ComChannel, ET.IORegister))

#: a set containing the possible types of acquirable elements.
#: Constant values belong to :class:`~sardana.sardanadefs.ElementType`
TYPE_ACQUIRABLE_ELEMENTS = set((ET.Motor, ET.CTExpChannel, ET.ZeroDExpChannel, \
    ET.OneDExpChannel, ET.TwoDExpChannel, \
    ET.ComChannel, ET.IORegister, ET.PseudoMotor, \
    ET.PseudoCounter))

#: a set containing the possible types of experimental channel elements.
#: Constant values belong to :class:`~sardana.sardanadefs.ElementType`
TYPE_EXP_CHANNEL_ELEMENTS = set((ET.CTExpChannel, ET.ZeroDExpChannel, \
    ET.OneDExpChannel, ET.TwoDExpChannel, ET.PseudoCounter))

#: a set containing the possible timer-able elements.
#: Constant values belong to :class:`~sardana.sardanadefs.ElementType`
TYPE_TIMERABLE_ELEMENTS = set((ET.CTExpChannel, ET.OneDExpChannel,
    ET.TwoDExpChannel))

#: a set containing the possible types of pseudo elements.
#: Constant values belong to :class:`~sardana.sardanadefs.ElementType`
TYPE_PSEUDO_ELEMENTS = set((ET.PseudoMotor, ET.PseudoCounter))

##: An enumeration describing the all possible sardana interfaces
#SardanaInterface = Enumeration("SardanaInterface", ( \
#    ("Object",            0b0000000000000001),
#    ("Element",           0b0000000000000011),
#    ("Class",             0b0000000000000101),
#    ("Library",           0b0000000000001001),
#    ("PoolObject",        0b0000000000010001),
#    ("PoolElement",       0b0000000000010011),
#    ("Pool",              0b0000000000110011),
#    ("Controller",        0b0000000001000001),
#    ("Moveable",          0b0000000010000001),
#    ("Acquirable",        0b0000000100000001),
#    ("Instrument",        0b0000001000000001),
#    ("Motor",             0b0000010000000001),
#    ("PseudoMotor",       0b0000100000000001),
#    ("IORegister",        0b0001000000000001),
#    ("ExpChannel",        0b0010000000000001),
#    ("CTExpChannel",      0b0100000000000001),
#    ("ZeroDExpChannel",   0b1000000000000001),
#    ("OneDExpChannel",    0b0000000000000001),
#    ("TwoDExpChannel",    0b0000000000000001),
#    ("PseudoCounter",     0b0000000000000001),
#    ("ComChannel",        0b0000000000000001),
#    ("MotorGroup",        0b0000000000000001),
#    ("MeasurementGroup",  0b0000000000000001),
#    ("ControllerLibrary", 0b0000000000000001),
#    ("ControllerClass",   0b0000000000000001),
#    ("Constraint",        0b0000000000000001),
#    ("External",          0b0000000000000001),

#    ("MacroServerObject", 0b0000000000000001),
#    ("MacroServerElement",0b0000000000000001),
#    ("MacroServer",       0b0000000000000001),
#    ("MacroLibrary",      0b0000000000000001),
#    ("MacroClass",        0b0000000000000001),
#    ("Macro",             0b0000000000000001), ) )

#: a dictionary containing the direct interfaces supported by each type
#: (:obj:`dict`\<:obj:`str`\, :obj:`tuple`\<:obj:`set`\<:obj:`str`\, :obj:`str`\>>>)
INTERFACES = {
    "Meta" : (set(), "A generic sardana meta object"),
    "Object" : (set(), "A generic sardana object"),
    "Element" : (set(("Object",)), "A generic sardana element"),
    "Class" : (set(("Object",)), "A generic sardana class"),
    "Function" : (set(("Object",)), "A generic sardana function"),
    "Library" : (set(("Object",)), "A generic sardana library"),
    "PoolObject" : (set(("Object",)), "A Pool object"),
    "PoolElement" : (set(("Element", "PoolObject")), "A Pool element"),
    "Pool" : (set(("PoolElement",)), "A Pool"),
    "Controller" : (set(("PoolElement",)), "A controller"),
    "Moveable" : (set(("PoolElement",)), "A moveable element"),
    "Acquirable" : (set(("PoolElement",)), "An acquirable element"),
    "Instrument" : (set(("PoolElement",)), "An instrument"),
    "Motor" : (set(("Moveable", "Acquirable")), "a motor"),
    "PseudoMotor" : (set(("Moveable", "Acquirable")), "A pseudo motor"),
    "IORegister" : (set(("Acquirable",)), "An IO register"),
    "ExpChannel" : (set(("Acquirable",)), "A generic experimental channel"),
    "CTExpChannel" : (set(("ExpChannel",)), "A counter/timer experimental channel"),
    "ZeroDExpChannel" : (set(("ExpChannel",)), "A 0D experimental channel"),
    "OneDExpChannel" : (set(("ExpChannel",)), "A 1D experimental channel"),
    "TwoDExpChannel" : (set(("ExpChannel",)), "A 2D experimental channel"),
    "PseudoCounter" : (set(("ExpChannel",)), "A pseudo counter"),
    "ComChannel" : (set(("PoolElement",)), "A communication channel"),
    "MotorGroup" : (set(("PoolElement",),), "A motor group"),
    "MeasurementGroup" : (set(("PoolElement",),), "A measurement group"),
    "ControllerLibrary" : (set(("Library", "PoolObject")), "A controller library"),
    "ControllerClass" : (set(("Class", "PoolObject")), "A controller class"),
    "Constraint" : (set(("PoolObject",)), "A constraint"),
    "External" : (set(("Object",)), "An external object"),

    "MacroServerObject" : (set(("Object",)), "A generic macro server object"),
    "MacroServerElement" : (set(("Element", "MacroServerObject")), "A generic macro server element"),
    "MacroServer" : (set(("MacroServerElement",)), "A MacroServer"),
    "Door" : (set(("MacroServerElement",)), "A macro server door"),
    "MacroLibrary" : (set(("Library", "MacroServerObject")), "A macro server library"),
    "MacroCode" : (set(("MacroServerObject",)), "A macro server macro code"),
    "MacroClass" : (set(("Class", "MacroCode")), "A macro server macro class"),
    "MacroFunction" : (set(("Function", "MacroCode")), "A macro server macro function"),
    "Macro" : (set(("MacroClass", "MacroFunction")), "A macro server macro"),

    "ParameterType" : (set(("Meta",)), "A generic macro server parameter type"),
}

#: a dictionary containing the *all* interfaces supported by each type
#: (:obj:`dict` <:obj:`str`, :obj:`set` < :obj:`str`> >)
INTERFACES_EXPANDED = {}

def __expand(name):
    direct_expansion, _ = INTERFACES[name]
    if isinstance(direct_expansion, (str, unicode)):
        direct_expansion = direct_expansion,
    exp = set(direct_expansion)
    for e in direct_expansion:
        e_value = INTERFACES_EXPANDED.get(e)
        if e_value is None:
            exp.update(__expand(e))
        else:
            exp.update(e_value[0])
    exp.add(name)
    return exp

def __build_interfaces_expanded():
    global INTERFACES_EXPANDED
    for i in INTERFACES:
        INTERFACES_EXPANDED[i] = __expand(i), INTERFACES[i][1]

__build_interfaces_expanded()

def __expand_sardana_interface_data(si_map, name, curr_id):
    if name in si_map:
        return curr_id
    d = 0
    i_expanded = set(INTERFACES_EXPANDED[name][0])
    i_expanded.remove(name)
    for interface in i_expanded:
        if interface not in si_map:
            curr_id = __expand_sardana_interface_data(si_map, interface, curr_id)
        d |= si_map[interface]
    si_map[name] = long(d | curr_id)
    return 2 * curr_id

def __root_expand_sardana_interface_data():
    curr_id = 1
    si_map = {}
    for interface in INTERFACES_EXPANDED:
        curr_id = __expand_sardana_interface_data(si_map, interface, curr_id)
    return si_map

#: An enumeration describing the all possible sardana interfaces
Interface = Enumeration("Interface",
                        __root_expand_sardana_interface_data().items())

def __create_sardana_interfaces():
    interfaces, interfaces_expanded = {}, {}
    for i in INTERFACES:
        i_enum = Interface[i]
        i_items, i_items_expanded = INTERFACES[i][0], INTERFACES_EXPANDED[i][0]
        i_enum_items = set(map(Interface.get, i_items))
        i_enum_items_expanded = set(map(Interface.get, i_items_expanded))
        interfaces[i_enum] = i_enum_items
        interfaces_expanded[i_enum] = i_enum_items_expanded
    return interfaces, interfaces_expanded

_Interfaces, _InterfacesExpanded = __create_sardana_interfaces()


#: a dictionary containing the direct interfaces supported by each type
#: (:obj:`dict` <:obj:`sardana.sardanadefs.Interface`, :obj:`set` < :obj:`sardana.sardanadefs.Interface`> >)
Interfaces = _Interfaces

#: a dictionary containing the *all* interfaces supported by each type.
#: (:obj:`dict` <:obj:`sardana.sardanadefs.Interface`, :obj:`set` < :obj:`sardana.sardanadefs.Interface`> >)
InterfacesExpanded = _InterfacesExpanded


class ScalarNumberFilter(object):
    """A simple scalar number filter that returns ``False`` if two numbers are
    indentical (i.e. |a-b| < error)"""

    def __call__(self, a, b):
        try:
            return math.fabs(a - b) > EpsilonError
        except:
            return a != b




from sardana.macroserver.macro import imacro, Type

@imacro()
def ask_number_of_points(self):
    """asks user for the number of points"""
    
    nb_points = self.input("How many points?", data_type=Type.Integer)
    

@imacro()
def ask_for_moveable(self):
    """asks user for a motor"""
    
    moveable = self.input("Which moveable?", data_type=Type.Moveable)
    self.output("You selected %s which is at %f", moveable, moveable.getPosition())

@imacro()
def ask_for_car_brand(self):
    """asks user for a car brand"""
    
    car_brands = "Mazda", "Citroen", "Renault"
    car_brand = self.input("Which car brand?", data_type=car_brands)
    self.output("You selected %s", car_brand)
    
@imacro()
def ask_for_multiple_car_brands(self):
    """asks user for several car brands"""
    
    car_brands = "Mazda", "Citroen", "Renault", "Ferrari", "Porche", "Skoda"
    car_brands = self.input("Which car brand(s)?", data_type=car_brands,
                            allow_multiple=True, title="Favorites")
    self.output("You selected %s", ", ".join(car_brands))

@imacro()
def ask_peak(self):
    """asks user for peak current of points with a custom title"""
    
    peak = self.input("What is the peak current?", data_type=Type.Float,
                      title="Peak selection")
    self.output("You selected a peak of %f A", peak)
    
@imacro()
def ask_peak_v2(self):
    """asks user for peak current of points with a custom title,
    default value, label and units"""
    
    label, unit = "peak", "mA"
    peak = self.input("What is the peak current?", data_type=Type.Float,
                      title="Peak selection", key=label, unit=unit,
                      default_value=123.4)
    self.output("You selected a %s of %f %s", label, peak, unit)
    
@imacro()
def ask_peak_v3(self):
    """asks user for peak current of points with a custom title,
    default value, label, units and ranges"""
    
    label, unit = "peak", "mA"
    peak = self.input("What is the peak current?", data_type=Type.Float,
                      title="Peak selection", key=label, unit=unit,
                      default_value=123.4, minimum=0.0, maximum=200.0)
    self.output("You selected a %s of %f %s", label, peak, unit)

@imacro()
def ask_peak_v4(self):
    """asks user for peak current of points with a custom title,
    default value, label, units, ranges and step size"""
    
    label, unit = "peak", "mA"
    peak = self.input("What is the peak current?", data_type=Type.Float,
                      title="Peak selection", key=label, unit=unit,
                      default_value=123.4, minimum=0.0, maximum=200.0,
                      step=5)
    self.output("You selected a %s of %f %s", label, peak, unit)
    
@imacro()
def ask_peak_v5(self):
    """asks user for peak current of points with a custom title,
    default value, label, units, ranges, step size and decimal places"""
    
    label, unit = "peak", "mA"
    peak = self.input("What is the peak current?", data_type=Type.Float,
                      title="Peak selection", key=label, unit=unit,
                      default_value=123.4, minimum=0.0, maximum=200.0,
                      step=5, decimals=2)
    self.output("You selected a %s of %f %s", label, peak, unit)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This package provides the spock macroserver connectivity"""

__all__ = ['GUIViewer', 'SpockBaseDoor', 'QSpockDoor', 'SpockDoor',
           'SpockMacroServer']

import os

import PyTango

from taurus.core import TaurusEventType, TaurusSWDevState

from sardana.sardanautils import is_pure_str, is_non_str_seq
from sardana.spock import genutils
from sardana.spock.inputhandler import SpockInputHandler, InputHandler

CHANGE_EVTS = TaurusEventType.Change, TaurusEventType.Periodic


if genutils.get_gui_mode() == 'qt':
    from sardana.taurus.qt.qtcore.tango.sardana.macroserver import QDoor, QMacroServer
    BaseDoor = QDoor
    BaseMacroServer = QMacroServer
    BaseGUIViewer = object
else:
    from sardana.taurus.core.tango.sardana.macroserver import BaseDoor, BaseMacroServer
    BaseGUIViewer = object


class GUIViewer(BaseGUIViewer):

    def __init__(self, door=None):
        BaseGUIViewer.__init__(self)
        self._door = door

    def run(self):
        self.plot()

    def show_scan(self, scan_nb=None, scan_history_info=None, directory_map=None):
        if scan_nb is None and scan_history_info is None:
            #===================================================================
            ##Hack to avoid ipython-qt issues. See similar workaround in expconf magic command
            ## @todo: do this in a better way
            #import taurus.qt.qtgui.plot
            #w = taurus.qt.qtgui.plot.TaurusTrend()
            #w.model = "scan://" + self._door.getNormalName()
            #w.show()
            import subprocess
            args = ['taurustrend', 'scan://%s' % self._door.getNormalName()]
            subprocess.Popen(args)
            #===================================================================
            return

        scan_dir, scan_file = None, None
        if scan_nb is None:
            for scan in reversed(scan_history_info):
                scan_dir = scan.get('ScanDir')
                scan_file = scan.get('ScanFile')
                if scan_dir is None or scan_file is None:
                    continue
                if not isinstance(scan_file, (str, unicode)):
                    scan_file = scan_file[0]
                break
            else:
                print "Cannot plot scan:"
                print "No scan in scan history was saved into a file"
                return
        else:
            for scan in reversed(scan_history_info):
                if scan['serialno'] == scan_nb:
                    scan_dir = scan.get('ScanDir')
                    scan_file = scan.get('ScanFile')
                    if scan_dir is None or scan_file is None:
                        print "Cannot plot scan:"
                        print "Scan %d was not saved into a file" % (scan_nb,)
                        return
                    if not isinstance(scan_file, (str, unicode)):
                        scan_file = scan_file[0]
                    break
            else:
                print "Cannot plot scan:"
                print "Scan %d not found in scan history" % (scan_nb,)
                return

        remote_file = os.path.join(scan_dir, scan_file)

        locations = [scan_dir]
        local_file = None
        if directory_map is None or not scan_dir in directory_map:
            if os.path.isdir(scan_dir):
                if scan_file in os.listdir(scan_dir):
                    local_file = remote_file
        else:
            local_directories = directory_map[scan_dir]
            if isinstance(scan_file, (str, unicode)):
                local_directories = [local_directories]
            locations = local_directories
            if scan_dir not in locations: locations.append(scan_dir)
            for local_directory in local_directories:
                if os.path.isdir(local_directory):
                    if scan_file in os.listdir(local_directory):
                        local_file = os.path.join(local_directory, scan_file)
                        break
        if local_file is None:
            print "Cannot plot scan:"
            print "Could not find %s in any of the following locations:" % (scan_file,)
            print "\n".join(locations)
            return

        import taurus.qt.qtgui.extra_nexus
        taurus_nexus_widget = taurus.qt.qtgui.extra_nexus.TaurusNeXusBrowser()
        taurus_nexus_widget.setMinimumSize(800, 600)

        print "Trying to open local scan file %s..." % (local_file,)
        taurus_nexus_widget.openFile(local_file)
        taurus_nexus_widget.show()
        nexus_widget = taurus_nexus_widget.neXusWidget()
        entry_name = "entry%d" % scan["serialno"]
        measurement_name = "%s/measurement" % entry_name
        title_name = "%s/title" % entry_name
        windowTitle = scan_file + "[" + entry_name + "]"

        try:
            #entry_index = taurus_nexus_widget.findNodeIndex(local_file, entry_name)
            measurement_index = taurus_nexus_widget.findNodeIndex(local_file, measurement_name)
            #nexus_widget.setRootIndex(entry_index)
            nexus_widget.setCurrentIndex(measurement_index)
            nexus_widget.expand(measurement_index)
            title_index = taurus_nexus_widget.findNodeIndex(local_file, title_name)
            file_model = nexus_widget.model()
            title = file_model.getNodeFromIndex(title_index)[0]
            windowTitle += " - " + title
        except Exception, e:
            print "Cannot plot scan:"
            print str(e)

        taurus_nexus_widget.setWindowTitle(windowTitle)

    def plot(self):
        try:
            import sps
        except:
            print 'sps module not available. No plotting'
            return

        try:
            import pylab
        except:
            print "pylab not available (try running 'spock -pylab'). No plotting"
            return

        door = genutils.get_door()

        try:
            env = dict(door.getEnvironmentObj().read().value)
        except Exception, e:
            print 'Unable to read environment. No plotting'
            print str(e)
            return

        program = door.getNormalName().replace('/', '').replace('_', '')
        try:
            array = env['ActiveMntGrp'].replace('/', '').replace('_', '').upper() + "0D"
            array_ENV = '%s_ENV' % array
        except:
            print 'ActiveMntGrp not defined. No plotting'
            return

        if not program in sps.getspeclist():
            print '%s not found. No plotting' % program
            return

        if not array in sps.getarraylist(program):
            print '%s not found in %s. No plotting' % (array, program)
            return

        if not array_ENV in sps.getarraylist(program):
            print '%s not found in %s. No plotting' % (array_ENV, program)
            return

        try:
            mem = sps.attach(program, array)
            mem_ENV = sps.attach(program, array_ENV)
        except Exception, e:
            print 'sps.attach error: %s. No plotting' % str(e)
            return

        # reconstruct the environment
        i, env = 0, {}
        while mem_ENV[i][0] != '':
            line = mem_ENV[i].tostring()
            eq, end = line.index('='), line.index('\x00')
            k, v = line[:eq], line[eq + 1:end]
            env[k] = v
            i += 1


        labels = env['axistitles'].split(' ')

        col_nb = len(labels)

        if col_nb < 4:
            print 'No data columns available in sps'
            return

        rows = int(env['nopts'])

        m = mem.transpose()

        x = m[1][:rows]
        colors = 'bgrcmyk'
        col_nb = min(col_nb, len(colors) + 3)
        # skip point_nb, motor and timer columns
        for i in xrange(3, col_nb):
            y = m[i][:rows]
            line, = pylab.plot(x, y, label=labels[i])
            line.linestyle = '-'
            line.linewidth = 1
            line.color = colors[i - 3]
        pylab.legend()


class SpockBaseDoor(BaseDoor):
    """A CLI version of the Door device"""

    console_editors = 'vi', 'vim', 'nano', 'joe', 'pico', 'emacs'

    Critical = 'Critical'
    Error = 'Error'
    Info = 'Info'
    Warning = 'Warning'
    Output = 'Output'
    Debug = 'Debug'
    Result = 'Result'
    RecordData = 'RecordData'

    MathFrontend = "matplotlib"

    def __init__(self, name, **kw):
        self._consoleReady = kw.get("consoleReady", False)
        if not kw.has_key('silent'): kw['silent'] = False
        self._lines = []
        self._spock_state = None
        self._plotter = GUIViewer(self)
        self.call__init__(BaseDoor, name, **kw)

    def create_input_handler(self):
        return SpockInputHandler(self)

    def get_color_mode(self):
        return genutils.get_color_mode()

    def _get_macroserver_for_door(self):
        ret = genutils.get_macro_server()
        return ret

    def _preprocessParameters(self, parameters):
        if is_pure_str(parameters):
            inside_str = False
            pars = []
            par = ''
            for c in parameters:
                if c == '"':
                    if inside_str:
                        inside_str = False
                        pars.append(par)
                        par = ''
                    else:
                        inside_str = True
                elif c == ' ':
                    if inside_str:
                        par += c
                    else:
                        pars.append(par)
                        par = ''
                else:
                    par += c
            if par: pars.append(par)
            return pars
        elif is_non_str_seq(parameters):
            return parameters

    def preRunMacro(self, obj, parameters):
        return BaseDoor.preRunMacro(self, obj, self._preprocessParameters(parameters))

    def runMacro(self, obj, parameters=[], synch=False):
        return BaseDoor.runMacro(self, obj, parameters=parameters, synch=synch)

    def _runMacro(self, xml, **kwargs):
        #kwargs like 'synch' are ignored in this re-implementation
        if self._spock_state != TaurusSWDevState.Running:
            print "Unable to run macro: No connection to door '%s'" % self.getSimpleName()
            raise Exception("Unable to run macro: No connection")
        if xml is None:
            xml = self.getRunningXML()
        kwargs['synch'] = True
        try:
            return BaseDoor._runMacro(self, xml, **kwargs)
        except KeyboardInterrupt:
            self.write('\nCtrl-C received: Stopping... ')
            self.block_lines = 0
            self.command_inout("StopMacro")
            self.writeln("Done!")
        except PyTango.DevFailed, e:
            if is_non_str_seq(e.args) and \
               not isinstance(e.args, (str, unicode)):
                reason, desc = e.args[0].reason, e.args[0].desc
                macro_obj = self.getRunningMacro()
                if reason == 'MissingParam':
                    print "Missing parameter:", desc
                    print macro_obj.getInfo().doc
                elif reason == 'WrongParam':
                    print "Wrong parameter:", desc
                    print macro_obj.getInfo().doc
                elif reason == 'UnkownParamObj':
                    print "Unknown parameter:", desc
                elif reason == 'MissingEnv':
                    print "Missing environment:", desc
                elif reason in ('API_CantConnectToDevice', 'API_DeviceNotExported'):
                    self._updateState(self._old_sw_door_state, TaurusSWDevState.Shutdown, silent=True)
                    print "Unable to run macro: No connection to door '%s'" % self.getSimpleName()
                else:
                    print "Unable to run macro:", reason, desc

    def _getMacroResult(self, macro):
        ret = None
        if macro.info.hasResult():
            ret = macro.getResult()

            if ret is None:
                return None

            if macro.info.getResult().type == 'File':

                commit_cmd = macro.info.hints['commit_cmd']

                if commit_cmd == None:
                    return ret

                local_f_name = ret[0]
                remote_f_name = ret[1]
                line_nb = ret[3]
                commit = CommitFile(commit_cmd, local_f_name, remote_f_name)
                self.pending_commits.update({ remote_f_name : commit })
                ip = genutils.get_ipapi()
                editor = genutils.get_editor()

                cmd = 'edit -x -n %s %s' % (line_nb, local_f_name)
                if not editor in self.console_editors:
                    cmd = 'bg _ip.magic("' + cmd + '")'
                ip.magic(cmd)
                # The return value of the macro was saved in a file and opened
                # with edit so we don't return anything to avoid big outputs
                # to the console
                ret = None
        return ret

    def plot(self):
        self._plotter.run()

    def show_scan(self, scan_nb=None, online=False):
        if online:
            self._plotter.show_scan()
            return
        env = self.getEnvironment()
        scan_history_info = env.get("ScanHistory")
        directory_map = env.get("DirectoryMap")
        self._plotter.show_scan(scan_nb=scan_nb,
                                scan_history_info=scan_history_info,
                                directory_map=directory_map)

    def stateChanged(self, s, t, v):
        old_sw_state = self._old_sw_door_state
        BaseDoor.stateChanged(self, s, t, v)
        new_sw_state = self._old_sw_door_state
        self._updateState(old_sw_state, new_sw_state)

    def _updateState(self, old_sw_state, new_sw_state, silent=False):
        user_ns = genutils.get_ipapi().user_ns
        if new_sw_state == TaurusSWDevState.Running:
            user_ns['DOOR_STATE'] = ""
        else:
            user_ns['DOOR_STATE'] = " (OFFLINE)"

        if not self.isConsoleReady():
            self._spock_state = new_sw_state
            return

        ss = self._spock_state
        if ss is not None and ss != new_sw_state and not silent:
            if ss == TaurusSWDevState.Running:
                self.write_asynch("\nConnection to door '%s' was lost.\n" % self.getSimpleName())
            elif new_sw_state == TaurusSWDevState.Running:
                self.write_asynch("\nConnection to the door (%s) has " \
                    "been restablished\n" % self.getSimpleName())
        self._spock_state = new_sw_state

    def write_asynch(self, msg):
        self._lines.append(msg)

    def pre_prompt_hook(self, ip):
        self._flush_lines()

    def _flush_lines(self):
        for l in self._lines:
            self.write(l)
        self._lines = []

    def setConsoleReady(self, state):
        self._consoleReady = state

    def isConsoleReady(self):
        return self._consoleReady

    def write(self, msg, stream=None):
        if not self.isConsoleReady():
            return
        return BaseDoor.write(self, msg, stream=stream)

    def processRecordData(self, data):
        if data is None: return
        data = data[1]
        if data['type'] == 'function':
            func_name = data['func_name']
            if func_name.startswith("pyplot."):
                func_name = self.MathFrontend + "." + func_name
            args = data['args']
            kwargs = data['kwargs']

            members = func_name.split(".")
            mod_list, fname = members[:-1], members[-1]
            mod_name = ".".join(mod_list)
            if mod_name:
                mod = __import__(mod_name, fromlist=mod_list)
                func = getattr(mod, fname)
            else:
                func = __builtins__[fname]
            try:
                func(*args, **kwargs)
            except Exception as e:
                self.logReceived(self.Warning, ['Unable to execute %s: ' % func_name, str(e)])

    _RECORD_DATA_THRESOLD = 4 * 1024 * 1024  # 4Mb

    def _processRecordData(self, data):
        if data is None: return
        value = data.value
        size = len(value[1])
        if size > self._RECORD_DATA_THRESOLD:
            sizekb = size / 1024
            self.logReceived(self.Info, ['Received long data record (%d Kb)' % sizekb,
                'It may take some time to process. Please wait...'])
        return BaseDoor._processRecordData(self, data)


from taurus.external.qt import Qt

class QSpockDoor(SpockBaseDoor):

    def __init__(self, name, **kw):
        self.call__init__(SpockBaseDoor, name, **kw)

        Qt.QObject.connect(self, Qt.SIGNAL('recordDataUpdated'),
                           self.processRecordData)

    def recordDataReceived(self, s, t, v):
        if genutils.get_pylab_mode() == "inline":
            if t not in CHANGE_EVTS: return
            res = BaseDoor.recordDataReceived(self, s, t, v)
            self.processRecordData(res)
        else:
            res = SpockBaseDoor.recordDataReceived(self, s, t, v)
        return res

    def create_input_handler(self):
        return InputHandler()


class SpockDoor(SpockBaseDoor):

    def _processRecordData(self, data):
        data = SpockBaseDoor._processRecordData(self, data)
        return self.processRecordData(data)


class SpockMacroServer(BaseMacroServer):
    """A CLI version of the MacroServer device"""

    def __init__(self, name, **kw):
        self._local_magic = {}
        self._local_var = set()
        self.call__init__(BaseMacroServer, name, **kw)

    def on_elements_changed(self, evt_src, evt_type, evt_value):
        return BaseMacroServer.on_elements_changed(self, evt_src, evt_type,
                                                   evt_value)

    _SKIP_ELEMENTS = 'controller', 'motorgroup', 'instrument', \
        'controllerclass', 'controllerlib', 'macrolib'

    def _addElement(self, element_data):
        element = BaseMacroServer._addElement(self, element_data)
        elem_type = element.type
        if "MacroCode" in element.interfaces:
            self._addMacro(element)
        elif elem_type not in self.NO_CLASS_TYPES:
            # TODO: when it becomes possible to do:
            # some taurus.Device.<attr name> = <value>
            # replace device_proxy with element
            device_proxy = element.getObj().getHWObj()
            genutils.expose_variable(element.name, device_proxy)
        return element

    def _removeElement(self, element_data):
        element = BaseMacroServer._removeElement(self, element_data)
        elem_type = element.type
        if "MacroCode" in element.interfaces:
            self._removeMacro(element)
        elif elem_type not in self.NO_CLASS_TYPES:
            genutils.unexpose_variable(element.name)
        return element

    def _addMacro(self, macro_info):
        macro_name = str(macro_info.name)

        def macro_fn(shell, parameter_s='', name=macro_name):
            parameters = genutils.arg_split(parameter_s, posix=True)
            door = genutils.get_door()
            door.runMacro(macro_name, parameters, synch=True)
            macro = door.getLastRunningMacro()
            if macro is not None:  # maybe none if macro was aborted
                return macro.getResult()

        macro_fn.func_name = macro_name
        macro_fn.__doc__ = macro_info.doc

        # register magic command
        genutils.expose_magic(macro_name, macro_fn)
        self._local_magic[macro_name] = macro_fn

        return macro_info

    def _removeMacro(self, macro_info):
        macro_name = macro_info.name
        genutils.unexpose_magic(macro_name)
        del self._local_magic[macro_name]



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""
channelWidgets.py: 
"""

__all__ = ["PoolChannel", "PoolChannelTV"]

import taurus
from taurus.external.qt import Qt
from taurus.qt.qtgui.panel import TaurusValue, TaurusDevButton
from taurus.qt.qtgui.container import TaurusWidget
from poolmotor import LabelWidgetDragsDeviceAndAttribute


class _ParentDevButton(TaurusDevButton):
    '''A TaurusDevButton that receives an attribute name but sets 
    the corresponding device as model. **For internal use only** '''
    def __init__(self, **kwargs):
        TaurusDevButton.__init__(self, **kwargs)
        self.setText('')
        self.setSizePolicy(Qt.QSizePolicy.Preferred, Qt.QSizePolicy.Maximum)

    def setModel(self, model):
        try: attr = taurus.Attribute(model)
        except: return
        dev = attr.getParentObj()
        devname = dev.getFullName()
        TaurusDevButton.setModel(self, devname)


class PoolChannelTV(TaurusValue):
    ''' A widget that displays and controls a pool channel device.
    It differs from :class:`PoolChannel` in that it behaves as a TaurusValue 
    (i.e., it allows its subwidgets to be aligned in columns in a TaurusForm)`
    '''
    def __init__(self, parent=None, designMode=False):
        TaurusValue.__init__(self, parent=parent, designMode=designMode)
        self.setLabelWidgetClass("LabelWidgetDragsDeviceAndAttribute")
        self.setLabelConfig('dev_alias')

    def getDefaultExtraWidgetClass(self):
        return _ParentDevButton

    def setModel(self, model):
        if model is not None:
            model = "%s/value" % model  #@todo: change this (it assumes tango naming!)
        TaurusValue.setModel(self, model)

    def showEvent(self, event):
        TaurusValue.showEvent(self, event)
        try: self.getModelObj().getParentObj().getAttribute('Value').enablePolling(force=True)
        except: pass

    def hideEvent(self, event):
        TaurusValue.hideEvent(self, event)
        try: self.getModelObj().getParentObj().getAttribute('Value').disablePolling()
        except: pass



class PoolChannel(TaurusWidget):
    ''' A widget that displays and controls a pool channel device
    
    .. seealso:: :class:`PoolChannelTV`
    '''
    def __init__(self, parent=None, designMode=False):
        TaurusWidget.__init__(self, parent)

        self.setLayout(Qt.QHBoxLayout())

        #put a widget with a TaurusValue
        w = Qt.QWidget()
        w.setLayout(Qt.QGridLayout())
        self._TaurusValue = TaurusValue(parent=w, designMode=designMode)
        self._TaurusValue.setLabelWidgetClass(LabelWidgetDragsDeviceAndAttribute)
        self._TaurusValue.setLabelConfig('dev_alias')
        self.layout().addWidget(w)

        #...and a dev button next to the widget
        self._devButton = TaurusDevButton(parent=self, designMode=designMode)
        self._devButton.setText('')
        self.layout().addWidget(self._devButton)

        self.connect(self, Qt.SIGNAL('modelChanged(const QString &)'), self._updateTaurusValue)

    def _updateTaurusValue(self):
        m = self.getModelName()
        self._TaurusValue.setModel("%s/value" % m)
        self._devButton.setModel(m)


#if __name__ == '__main__':
#    import sys
#    app = Qt.QApplication(sys.argv)
#
#    form = PoolChannel()
#
#    #model = 'tango://controls02:10000/expchan/bl97_simucotictrl_1/1'
#    model = 'ct_cp1_1'
#    if len(sys.argv)>1:
#        model = sys.argv[1]
#    form.setModel(model)
#
#
#    form.show()
#    sys.exit(app.exec_())



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""This module is part of the Python Pool libray. It defines the base classes
for"""

__all__ = ["PoolMotor"]

__docformat__ = 'restructuredtext'

import time
import math

from sardana import EpsilonError, State, ElementType
from sardana.sardanaattribute import SardanaAttribute, ScalarNumberAttribute, \
    SardanaSoftwareAttribute
from sardana.sardanaevent import EventType
from sardana.sardanautils import assert_type, is_number
from sardana.pool.poolelement import PoolElement
from sardana.pool.poolmotion import PoolMotion, MotionState


class Position(SardanaAttribute):

    def __init__(self, *args, **kwargs):
        super(Position, self).__init__(*args, **kwargs)
        self.get_offset().add_listener(self.on_change)
        self.get_sign().add_listener(self.on_change)
        self.get_dial().add_listener(self.on_change)

    def get_dial(self):
        return self.obj.get_dial_position_attribute()

    def get_offset(self):
        return self.obj.get_offset_attribute()

    def get_sign(self):
        return self.obj.get_sign_attribute()

    def _in_error(self):
        return self.get_dial().in_error()

    def _has_value(self):
        return self.get_dial().has_value()

    def _has_write_value(self):
        return self.get_dial().has_write_value()

    def _get_value(self):
        return self.calc_position()

    def _get_write_value(self):
        dial = self.get_dial().get_write_value()
        return self.calc_position(dial=dial)

    def _set_value(self, value, exc_info=None, timestamp=None, propagate=1):
        raise Exception("Cannot set position value for %s" % self.obj.name)

    def _set_write_value(self, w_value, timestamp=None, propagate=1):
        # let the write value be stored by dial using the current offset and
        # sign. This way, retrieving the write value is done in reverse applying
        # the offset and sign in use at that time
        w_dial = self.calc_dial_position(w_value)
        self.get_dial().set_write_value(w_dial, timestamp=timestamp, propagate=0)
        self.fire_write_event(propagate=propagate)

    def _get_exc_info(self):
        return self.get_dial().get_exc_info()

    def _get_timestamp(self):
        return self.get_dial().get_timestamp()

    def _get_write_timestamp(self):
        return self.get_dial().get_write_timestamp()

    def calc_position(self, dial=None):
        """Returns the computed position from last the dial position from the
        given parameter or (if None), the last dial position obtained from
        hardware read. 
        
        :param dial: the new dial position [default: None, meaning use the
                     current dial position.
        :return: the computed user position
        :rtype: obj
        
        :raises:
            :exc:`Exception` if dial_position is None and no read value has
            been set yet"""
        obj = self.obj
        if dial is None:
            dial_attr = obj.dial_position
            if dial_attr.in_error():
                raise dial_attr.exc_info[1]
            dial = dial_attr.value
        if not is_number(dial):
            raise Exception("Controller returns not a number %s" % dial)
        sign, offset = obj.sign.value, obj.offset.value
        return sign * dial + offset

    def calc_dial_position(self, position=None):
        """Returns the dial position for the  given position. If position is
        not given (or is None) it uses this object's *write* value.
        
        :param position:
            the position to be converted to dial [default: None meaning use the
            this attribute's *write* value
        :type position: obj
        :return: the computed dial position
        :rtype: obj"""
        obj = self.obj
        if position is None:
            position = self.w_value
        sign, offset = obj.sign.value, obj.offset.value
        return (position - offset) / sign

    def calc_motion(self, new_position):
        """Calculate the motor position, dial position, backlash for the
        given final position."""
        obj = self.obj
        ctrl = obj.controller
        step_per_unit = obj._step_per_unit
        backlash = obj._backlash

        # compute dial position
        new_dial = self.calc_dial_position(new_position)

        # add backlash if necessary
        do_backlash = False

        if obj.has_backlash() and not ctrl.has_backlash():
            dial_attr = self.get_dial()

            if dial_attr.in_error():
                raise dial_attr.get_exc_info()[1]

            old_dial = dial_attr.value
            displacement = new_dial - old_dial

            if math.fabs(displacement) > EpsilonError:
                positive_displacement = displacement > 0
                positive_backlash = self.is_backlash_positive()
                do_backlash = positive_backlash != positive_displacement
                if do_backlash:
                    new_dial = new_dial - backlash / step_per_unit

        # compute a rounding value if necessary
        if ctrl.wants_rounding():
            nb_step = round(new_dial * step_per_unit)
            new_dial = nb_step / step_per_unit

        backlash_position = new_dial
        if do_backlash:
            backlash_position = new_dial + backlash / step_per_unit

        return new_position, new_dial, do_backlash, backlash_position

    def on_change(self, evt_src, evt_type, evt_value):
        self.fire_read_event(propagate=evt_type.priority)

    def update(self, cache=True, propagate=1):
        self.get_dial().update(cache=cache, propagate=propagate)


class DialPosition(ScalarNumberAttribute):

    def update(self, cache=True, propagate=1):
        if not cache or not self.has_value():
            dial_position_value = self.obj.read_dial_position()
            self.set_value(dial_position_value, propagate=propagate)


class LimitSwitches(ScalarNumberAttribute):
    pass


class Offset(SardanaSoftwareAttribute):
    pass


class Sign(SardanaSoftwareAttribute):
    pass


class PoolMotor(PoolElement):
    """An internal Motor object. **NOT** part of the official API. Accessing
    this object from a controller plug-in may lead to undetermined behavior
    like infinite recursion."""

    def __init__(self, **kwargs):
        kwargs['elem_type'] = ElementType.Motor
        PoolElement.__init__(self, **kwargs)
        on_change = self.on_change
        self._offset = Offset(self, initial_value=0, listeners=on_change)
        self._sign = Sign(self, initial_value=1, listeners=on_change)
        self._dial_position = DialPosition(self, listeners=on_change)
        self._position = Position(self, listeners=on_change)
        self._backlash = 0
        self._step_per_unit = 1.0
        self._limit_switches = LimitSwitches(self, name="Limit_switches",
                                             initial_value=3 * (False,),
                                             listeners=on_change)
        self._acceleration = None
        self._deceleration = None
        self._velocity = None
        self._base_rate = None
        self._instability_time = None
        self._in_start_move = False
        motion_name = "%s.Motion" % self._name
        self.set_action_cache(PoolMotion(self, motion_name))

    # --------------------------------------------------------------------------
    # Event forwarding
    # --------------------------------------------------------------------------

    def on_change(self, evt_src, evt_type, evt_value):
        # forward all events coming from attributes to the listeners
        self.fire_event(evt_type, evt_value)

    # --------------------------------------------------------------------------
    # state information
    # --------------------------------------------------------------------------

    def _from_ctrl_state_info(self, state_info):
        state_info, _ = state_info

        try:
            state_str = State.whatis(state_info)
            return int(state_info), "{0} is in {1}".format(self.name, state_str), 0
        except KeyError:
            pass

        if len(state_info) > 2:
            state, status, ls = state_info[:3]
        else:
            state, other = state_info[:2]
            if is_number(other):
                ls, status = other, ''
            else:
                ls, status = 0, other
        state, ls = int(state), tuple(map(bool, (ls & 1, ls & 2, ls & 4)))
        return state, status, ls

    def _set_state_info(self, state_info, propagate=1):
        PoolElement._set_state_info(self, state_info, propagate=propagate)
        ls = state_info[-1]
        if self._sign.value < 0:
            ls = ls[0], ls[2], ls[1]
        self._set_limit_switches(ls, propagate=propagate)

    # --------------------------------------------------------------------------
    # state information
    # --------------------------------------------------------------------------

    _STD_STATUS = "{name} is {state}{limit_switches}{ctrl_status}"
    def calculate_state_info(self, state_info=None):
        if state_info is None:
            state = self._state
            status = self._status
            ls = self._limit_switches.value
        else:
            state, status, ls = state_info
        if state == State.On:
            state_str = "Stopped"
        elif state == State.Moving:
            state_str = "Moving"
            motion = self.get_operation()
            if motion is None:
                state_str += " (external)"
            else:
                motion_state = motion._motion_info[self].motion_state
                if motion_state == MotionState.MovingBacklash:
                    state_str += " (backlash)"
                elif motion_state == MotionState.MovingInstability:
                    state_str += " (instability)"
        else:
            state_str = "in " + State[state]

        limit_switches = ""
        if ls[0]:
            limit_switches += ". Hit home switch"
        if ls[1]:
            limit_switches += ". Hit upper switch"
        if ls[2]:
            limit_switches += ". Hit lower switch"

        if len(status) > 0:
            status = "\n" + status
        new_status = self._STD_STATUS.format(name=self.name, state=state_str,
                                             limit_switches=limit_switches,
                                             ctrl_status=status)
        return state, new_status, ls

    # --------------------------------------------------------------------------
    # limit switches
    # --------------------------------------------------------------------------

    def inspect_limit_switches(self):
        """returns the current (cached value of the limit switches

        :return: the current limit switches flags"""
        return self._limit_switches

    def get_limit_switches(self, cache=True, propagate=1):
        """Returns the motor limit switches state.

        :param cache:
            if ``True`` (default) return value in cache, otherwise read value
            from hardware
        :type cache:
            bool
        :param propagate:
            0 for not propagating, 1 to propagate, 2 propagate with priority
        :type propagate:
            int
        :return:
            the motor limit switches state
        :rtype:
            :class:`~sardana.sardanaattribute.SardanaAttribute`"""
        self.get_state(cache=cache, propagate=propagate)
        return self._limit_switches

    def set_limit_switches(self, ls, propagate=1):
        self._set_limit_switches(ls, propagate=propagate)

    def put_limit_switches(self, ls, propagate=1):
        self._limit_switches = tuple(ls)

    def _set_limit_switches(self, ls, propagate=1):
        self._limit_switches.set_value(tuple(ls), propagate=propagate)

    limit_switches = property(get_limit_switches, set_limit_switches,
                              doc="motor limit switches")

    # --------------------------------------------------------------------------
    # instability time
    # --------------------------------------------------------------------------

    def has_instability_time(self, cache=True):
        it = self._instability_time
        return it is not None and it > 0.0

    def get_instability_time(self, cache=True):
        return self._instability_time

    def set_instability_time(self, instability_time, propagate=1):
        self._instability_time = instability_time
        if propagate > 0:
            self.fire_event(EventType("instability_time", priority=propagate),
                            instability_time)

    instability_time = property(get_instability_time, set_instability_time,
                                doc="motor instability time")

    # --------------------------------------------------------------------------
    # backlash
    # --------------------------------------------------------------------------

    def has_backlash(self, cache=True):
        return self._backlash != 0

    def is_backlash_positive(self, cache=True):
        return self._backlash > 0

    def is_backlash_negative(self, cache=True):
        return self._backlash < 0

    def get_backlash(self, cache=True):
        return self._backlash

    def set_backlash(self, backlash, propagate=1):
        self._backlash = backlash
        if propagate > 0:
            self.fire_event(EventType("backlash", priority=propagate), backlash)

    backlash = property(get_backlash, set_backlash, doc="motor backlash")

    # --------------------------------------------------------------------------
    # offset
    # --------------------------------------------------------------------------

    def get_offset_attribute(self):
        return self._offset

    def get_offset(self, cache=True):
        return self._offset

    def set_offset(self, offset, propagate=1):
        self._offset.set_value(offset, propagate=propagate)

    offset = property(get_offset, set_offset, doc="motor offset")

    # --------------------------------------------------------------------------
    # sign
    # --------------------------------------------------------------------------

    def get_sign_attribute(self):
        return self._sign

    def get_sign(self, cache=True):
        return self._sign

    def set_sign(self, sign, propagate=1):
        old_sign = self._sign.value
        self._sign.set_value(sign, propagate=propagate)
        # invert lower with upper limit switches and send event in case of change
        ls = self._limit_switches
        if old_sign != sign and ls.has_value():
            value = ls.value
            value = value[0], value[2], value[1]
            self._set_limit_switches(value, propagate=propagate)

    sign = property(get_sign, set_sign, doc="motor sign")

    # --------------------------------------------------------------------------
    # step per unit
    # --------------------------------------------------------------------------

    def get_step_per_unit(self, cache=True, propagate=1):
        if not cache or self._step_per_unit is None:
            step_per_unit = self.read_step_per_unit()
            self._set_step_per_unit(step_per_unit, propagate=propagate)
        return self._step_per_unit

    def set_step_per_unit(self, step_per_unit, propagate=1):
        if step_per_unit <= 0.0:
            raise Exception("Step per unit must be > 0.0")
        self.controller.set_axis_par(self.axis, "step_per_unit", step_per_unit)
        self._set_step_per_unit(step_per_unit, propagate=propagate)

    def _set_step_per_unit(self, step_per_unit, propagate=1):
        self._step_per_unit = step_per_unit
        if propagate:
            self.fire_event(EventType("step_per_unit", priority=propagate), step_per_unit)
            # force ask controller for new position to send priority event
            self.get_position(cache=False, propagate=2)

    def read_step_per_unit(self):
        step_per_unit = self.controller.get_axis_par(self.axis, "step_per_unit")
        assert_type(float, step_per_unit)
        return step_per_unit

    step_per_unit = property(get_step_per_unit, set_step_per_unit,
                             doc="motor steps per unit")

    # --------------------------------------------------------------------------
    # acceleration
    # --------------------------------------------------------------------------

    def get_acceleration(self, cache=True, propagate=1):
        if not cache or self._acceleration is None:
            acceleration = self.read_acceleration()
            self._set_acceleration(acceleration, propagate=propagate)
        return self._acceleration

    def set_acceleration(self, acceleration, propagate=1):
        self.controller.set_axis_par(self.axis, "acceleration", acceleration)
        self._set_acceleration(acceleration, propagate=propagate)

    def _set_acceleration(self, acceleration, propagate=1):
        self._acceleration = acceleration
        if not propagate:
            return
        self.fire_event(EventType("acceleration", priority=propagate), acceleration)

    def read_acceleration(self):
        acceleration = self.controller.get_axis_par(self.axis, "acceleration")
        assert_type(float, acceleration)
        return acceleration

    acceleration = property(get_acceleration, set_acceleration,
                            doc="motor acceleration")

    # --------------------------------------------------------------------------
    # deceleration
    # --------------------------------------------------------------------------

    def get_deceleration(self, cache=True, propagate=1):
        if not cache or self._deceleration is None:
            deceleration = self.read_deceleration()
            self._set_deceleration(deceleration, propagate=propagate)
        return self._deceleration

    def set_deceleration(self, deceleration, propagate=1):
        self.controller.set_axis_par(self.axis, "deceleration", deceleration)
        self._set_deceleration(deceleration, propagate=propagate)

    def _set_deceleration(self, deceleration, propagate=1):
        self._deceleration = deceleration
        if not propagate:
            return
        self.fire_event(EventType("deceleration", priority=propagate), deceleration)

    def read_deceleration(self):
        deceleration = self.controller.get_axis_par(self.axis, "deceleration")
        assert_type(float, deceleration)
        return deceleration

    deceleration = property(get_deceleration, set_deceleration,
                            doc="motor deceleration")
    # --------------------------------------------------------------------------
    # base_rate
    # --------------------------------------------------------------------------

    def get_base_rate(self, cache=True, propagate=1):
        if not cache or self._base_rate is None:
            base_rate = self.read_base_rate()
            self._set_base_rate(base_rate, propagate=propagate)
        return self._base_rate

    def set_base_rate(self, base_rate, propagate=1):
        self.controller.set_axis_par(self.axis, "base_rate", base_rate)
        self._set_base_rate(base_rate, propagate=propagate)

    def _set_base_rate(self, base_rate, propagate=1):
        self._base_rate = base_rate
        if not propagate:
            return
        self.fire_event(EventType("base_rate", priority=propagate), base_rate)

    def read_base_rate(self):
        base_rate = self.controller.get_axis_par(self.axis, "base_rate")
        assert_type(float, base_rate)
        return base_rate

    base_rate = property(get_base_rate, set_base_rate,
                         doc="motor base rate")

    # --------------------------------------------------------------------------
    # velocity
    # --------------------------------------------------------------------------

    def get_velocity(self, cache=True, propagate=1):
        if not cache or self._velocity is None:
            velocity = self.read_velocity()
            self._set_velocity(velocity, propagate=propagate)
        return self._velocity

    def set_velocity(self, velocity, propagate=1):
        self.controller.set_axis_par(self.axis, "velocity", velocity)
        self._set_velocity(velocity, propagate=propagate)

    def _set_velocity(self, velocity, propagate=1):
        self._velocity = velocity
        if not propagate:
            return
        self.fire_event(EventType("velocity", priority=propagate), velocity)

    def read_velocity(self):
        velocity = self.controller.get_axis_par(self.axis, "velocity")
        assert_type(float, velocity)
        return velocity

    velocity = property(get_velocity, set_velocity,
                        doc="motor velocity")

    # --------------------------------------------------------------------------
    # position & dial position
    # --------------------------------------------------------------------------

    def define_position(self, position):
        dial = self.get_position_attribute().calc_dial_position(position)
        self.controller.define_position(self.axis, dial)
        # force an event with the new position
        self.get_position(cache=False, propagate=2)

    def get_position_attribute(self):
        """Returns the position attribute object for this motor
        
        :return: the position attribute
        :rtype: :class:`~sardana.sardanaattribute.SardanaAttribute`"""
        return self._position

    def get_position(self, cache=True, propagate=1):
        """Returns the user position.

        :param cache:
            if ``True`` (default) return value in cache, otherwise read value
            from hardware
        :type cache:
            bool
        :param propagate:
            0 for not propagating, 1 to propagate, 2 propagate with priority
        :type propagate:
            int
        :return:
            the user position
        :rtype:
            :class:`~sardana.sardanaattribute.SardanaAttribute`"""
        position = self._position
        position.update(cache=cache, propagate=propagate)
        return position

    def set_position(self, position):
        """Moves the motor to the specified user position

        :param position:
            the user position to move to
        :type position:
            :class:`~numbers.Number`"""
        self.start_move(position)

    def set_write_position(self, w_position, timestamp=None, propagate=1):
        """Sets a new write value for the user position.

        :param w_position:
            the new write value for user position
        :type w_position:
            :class:`~numbers.Number`
        :param propagate:
            0 for not propagating, 1 to propagate, 2 propagate with priority
        :type propagate:
            int"""
        self._position.set_write_value(w_position, timestamp=timestamp,
                                       propagate=propagate)

    def read_dial_position(self):
        """Reads the dial position from hardware.

        :return:
            a :class:`~sardana.sardanavalue.SardanaValue` containing the dial
            position
        :rtype:
            :class:`~sardana.sardanavalue.SardanaValue`"""
        return self.motion.read_dial_position(serial=True)[self]

    def put_dial_position(self, dial_position_value, propagate=1):
        """Sets a new dial position.

        :param dial_position_value:
            the new dial position value
        :type dial_position_value:
            :class:`~sardana.sardanavalue.SardanaValue`
        :param propagate:
            0 for not propagating, 1 to propagate, 2 propagate with priority
        :type propagate:
            int"""
        dp = self._dial_position
        dp.set_value(dial_position_value, propagate=propagate)
        return dp

    def get_dial_position_attribute(self):
        """Returns the dial position attribute object for this motor
        
        :return: the dial position attribute
        :rtype: :class:`~sardana.sardanaattribute.SardanaAttribute`"""
        return self._dial_position

    def get_dial_position(self, cache=True, propagate=1):
        """Returns the dial position.

        :param cache:
            if ``True`` (default) return value in cache, otherwise read value
            from hardware
        :type cache:
            bool
        :param propagate:
            0 for not propagating, 1 to propagate, 2 propagate with priority
        :type propagate:
            int
        :return:
            the dial position
        :rtype:
            :class:`~sardana.sardanaattribute.SardanaAttribute`"""
        dp = self._dial_position
        dp.update(cache=cache, propagate=propagate)
        return dp

    position = property(get_position, set_position, doc="motor user position")
    dial_position = property(get_dial_position, doc="motor dial position")

    # --------------------------------------------------------------------------
    # default acquisition channel
    # --------------------------------------------------------------------------

    def get_default_attribute(self):
        return self.get_position_attribute()

    # --------------------------------------------------------------------------
    # motion
    # --------------------------------------------------------------------------

    def get_motion(self):
        return self.get_action_cache()

    motion = property(get_motion, doc="motion object")

    # --------------------------------------------------------------------------
    # motion calculation
    # --------------------------------------------------------------------------

    def calculate_motion(self, new_position, items=None, calculated=None):
        """Calculate the motor position, dial position, backlash for the
        given final position. Items specifies the where to put the calculated
        values, calculated is not used by physical motors"""

        step_per_unit = self._step_per_unit
        backlash = self._backlash
        ctrl = self.controller
        pos_attr = self.get_position_attribute()

        # compute dial position
        new_dial = pos_attr.calc_dial_position(new_position)

        # add backlash if necessary
        do_backlash = False

        if self.has_backlash() and not ctrl.has_backlash():
            dial_attr = self.get_dial_position_attribute()

            if dial_attr.in_error():
                raise dial_attr.get_exc_info()[1]

            old_dial = dial_attr.value
            displacement = new_dial - old_dial

            if math.fabs(displacement) > EpsilonError:
                positive_displacement = displacement > 0
                positive_backlash = self.is_backlash_positive()
                do_backlash = positive_backlash != positive_displacement
                if do_backlash:
                    new_dial = new_dial - backlash / step_per_unit

        # compute a rounding value if necessary
        if ctrl.wants_rounding():
            nb_step = round(new_dial * step_per_unit)
            new_dial = nb_step / step_per_unit

        backlash_position = new_dial
        if do_backlash:
            backlash_position = new_dial + backlash / step_per_unit

        if items is None:
            items = {}
        items[self] = new_position, new_dial, do_backlash, backlash_position
        return items

    def start_move(self, new_position):
        self._in_start_move = True
        try:
            return self._start_move(new_position)
        finally:
            self._in_start_move = False

    def _start_move(self, new_position):
        if not self._simulation_mode:
            # update the dial value from the controller in case motor has been
            # moved outside sardana.
            # TODO: also update step_per_unit
            self.get_dial_position_attribute().update(cache=False, propagate=1)

            # calculate the motion (dial position, backlash, etc)
            items = self.calculate_motion(new_position)
            self.debug("Start motion user=%f, dial=%f, backlash? %s, "
                       "dial_backlash=%f", *items[self])

            timestamp = time.time()
            # update the write position
            self.set_write_position(items[self][0], timestamp=timestamp, propagate=0)

            # move!
            self.motion.run(items=items)



##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
## 
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
## 
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
## 
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""IORegister related macros"""

__docformat__ = 'restructuredtext'

__all__ = ["write_ioreg", "read_ioreg"]

from sardana.macroserver.macro import *

import array


class write_ioreg(Macro):
    """Writes a value to an input register"""
    
    param_def = [
       ['input/output register', Type.IORegister, None, 'input/output register'],
       ['data', Type.Integer, None, 'value to be send']
    ]
    
    def run(self, ioreg, data):
        name = ioreg.getName()
        o = "Writing " + str(data) + " to " + name + " register "
        self.output(o)
        data = ioreg.writeIORegister(data)



class read_ioreg(Macro):
    """Reads an output register"""
    
    param_def = [
       ['input/output register', Type.IORegister, None, 'input/output register']
    ]
    
    def run(self, ioreg):
        name = ioreg.getName()
        data = ioreg.readIORegister(force=True)
        o = "Reading " +  name + " register "
        self.output(o)
        self.output(data)



#!/usr/bin/env python

##############################################################################
##
## This file is part of Sardana
##
## http://www.tango-controls.org/static/sardana/latest/doc/html/index.html
##
## Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
## Sardana is free software: you can redistribute it and/or modify
## it under the terms of the GNU Lesser General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## Sardana is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public License
## along with Sardana.  If not, see <http://www.gnu.org/licenses/>.
##
##############################################################################

"""

Sardana includes a framework allowing to test its features.
This framework is aimed to be used with the objective of finding bugs and
promote test driven development. \n

This Framework is the outcome of the Sardana Enhancement Proposal 5 (SEP5). Its
documentation is located here:
http://sourceforge.net/p/sardana/wiki/SEP5/ \n

Ideally, each bug found should be accompanied by a test revealing the bug.
That eases the process of correcting bugs. \n

The objective of the Sardana Test Framework is not to provide a test for
every piece of code, but rather to ease collaborations and allow people
from other institutions to provide their own test classes. \n

At the moment the Sardana testing is focused on Unit Tests, but in the future
the objective is to extend it to integration and system tests as well. \n

In order to run all the tests provided by Sardana test framework, at a given
moment, just go to 'sardana/src/sardana/test' and execute: \n
python testsuite.py

Sardana Test Framework is based on 'unittest'. This means that each of the
Sardana python test classes inherits from unittest.TestCase. \n

Each of the Sardana Tests has to be acompanied by a documentation that is
written in the module, class and method docstrings; as well as in the assert
methods. \n

All tests have to be written in folders named 'test/'. \n

Sardana Test Framework provides tools for testing macros. These tools comes
from: \n
* sardana/src/sardana/macroserver/macros/test/base.py
* sardana/src/sardana/macroserver/macros/test/macroexecutor.py
* sardana/src/sardana/macroserver/macros/test/sardemoenv.py
* sardana/src/sardana/tango/macroserver/test/macroexecutor.py

Tests of macros are done using the motors and experimental channels created
by Sardana demo. \n

base.py provides the mean of executing macros and test the function Run and
Stop for each macro. Macro test classes can inherit from RunMacroTestCase,
RunStopMacroTestCase or BaseMacroTestCase. \n

Macros as 'lsm' inherits from RunMacroTestCase as it is interesting to test
if the macros can be executed. The test methods can override the function
macro_runs() that is defined in the class RunMacroTestCase. \n

However, scan macros inherits from RunStopMacroTestCase as it is interesting
to test both: if the macros can be executed and if they can be aborted. \n

Another capacity provided by the Framework is the option to execute the
same test method with many different macro input parameters. These is done
by the help of a decorator inserted at the beginning of the test function.
One decorator has to be used for each set of macro input parameters. Examples
of the decorator usage can be seen in: test_scan.py \n

If new tests of scan macros or list macros have to be added, that can be
done in test_scan.py or in test_list.py where a useful base class is
provided. \n


Examples of Sardana tests using tools of Sardana Test Framework are: \n
* sardana/src/sardana/test/test_sardanavalue.py
* sardana/src/sardana/test/test_parameter.py
* sardana/src/sardana/macroserver/macros/test_ct.py
* sardana/src/sardana/macroserver/macros/test_list.py
* sardana/src/sardana/macroserver/macros/test_wm.py

"""
